#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Azure Handler to connect and publish the data.
To use, simply 'import AzureHandler' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com); Sachin (spatil@a10networks.com)'

import hashlib
import hmac
import base64
import json
import socket
from common.toa_constant import Constant
from common.toa_utils import Utils
from common.toa_logging import Logging
from concurrent.futures import ThreadPoolExecutor
import concurrent.futures

class AzureHandler:

    _logger = Logging().get_logger("AzureHandler")
    _credentials=None
    _thunder=None
    _token=None
    _url=None
    _vm=None
 
    def __init__(self, thunder, token):
        self._thunder=thunder
        self._credentials=Utils._azure_credentials 
        self._token=token

    def token(self, resource):
        
        client_id = self._credentials.get(Constant._AZURE_CLIENT_ID)
        secret_id = self._credentials.get(Constant._AZURE_SECRET_ID)
        tenant_id = self._credentials.get(Constant._AZURE_TENANT_ID)

        _headers = {
            Constant._CONTENT_TYPE: Constant._APPLICATION_URLENCODED
        }
         
        _body = {'grant_type': 'client_credentials', 'client_id': f'{client_id}',
                   'client_secret': f'{secret_id}', 'resource': 
                       {resource}}
        
        response = Utils.rest_api(Constant._AZURE_OAUTH2_URL.format(tenant_id), _headers, _body, Constant._POST, Constant._ERROR_AZURE_AUTH_RESPONSE, self._thunder.get(Constant._IP))
        self._token = json.loads(response.text).get('access_token')
        return self._token

    def publish_metric(self, data):
        
        if (Utils.is_valid(data) and len(data.keys()) > 0):
            metric_name = data.__iter__().__next__()
            
            dimention = self._thunder.get(Constant._RESOURCE_ID, socket.gethostname())

            _body = json.dumps({
                "time": Utils._datetime_str,
                "data": {
                    "baseData": {
                        "metric": metric_name,
                        "namespace":self._thunder.get(Constant._NAMESPACE),
                        "dimNames": [
                            metric_name
                        ],
                        "series": [
                            {
                                "dimValues": [
                                    dimention
                                ],
                                "min": 0,
                                "max": 0,
                                "sum": data[metric_name],
                                "count": 1
                            }
                        ]
                    }
                }
            })
 
            response = Utils.rest_api(Constant._AZURE_METRICS_URL.format(self._credentials.get(Constant._AZURE_LOCATION), Utils._config.get(Constant._AZURE_METRIC_RESOURCE_ID)), Utils.get_headers(self._token), _body, Constant._POST, Constant._ERROR_AZURE_METRICS, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(Utils.is_valid(response)):
                return data
            else:
                return None
        else:
            return None
                
    def publish_log(self, data):
        if(not Utils.is_valid(data)):
            return None
        
        if(len(data)>0):
            generator = self.format(data)
            logs = []
            for log in generator:
                logs.append(log)
            def key(log):
                x_headers = 'x-ms-date:' + Utils._datetime_gmt_str
                customer_id = Utils._config.get(Constant._AZURE_LOG_WORKSPACE_ID)
                shared = self._credentials.get(Constant._AZURE_WORKSPACE_PRIMARY_KEY)
                string_to_hash = Constant._POST + "\n" + str(len(log)) + "\n" + \
                                 Constant._APPLICATION_JSON + "\n" + x_headers + "\n" + '/api/logs'
                bytes_to_hash = bytes(string_to_hash, encoding="utf-8")
                decoded = base64.b64decode(shared)
                encoded_hash = base64.b64encode(
                    hmac.new(decoded, bytes_to_hash,
                             digestmod=hashlib.sha256).digest()).decode()
                return "SharedKey {}:{}".format(customer_id, encoded_hash)
        
            _headers = {
                Constant._CONTENT_TYPE: Constant._APPLICATION_JSON,
                Constant._AUTHORIZATION: key(json.dumps(data)),
                'Log-Type': "THUNDER_SYSLOG",
                'x-ms-date': Utils._datetime_gmt_str,
                'x-ms-AzureResourceId': self._thunder.get(Constant._RESOURCE_ID)
            }
            
            response = Utils.rest_api(Constant._AZURE_LOGS_URL.format(Utils._config.get(Constant._AZURE_LOG_WORKSPACE_ID)), _headers, json.dumps(data), Constant._POST, Constant._ERROR_AZURE_LOGS, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if (Utils.is_valid(response)):
                self._logger.info(Constant._SUCCESS_LOG.format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper(), len(data)))
                return Constant._SUCCESS
        else:
            self._logger.info((Constant._SKIP_LOG).format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper()))
        return None
        
    
    def format(self, data): 
        if isinstance(data, str):
            data = json.loads(data)

        # replace key log-data with text
        # add meta data to search logs on vRLI UI
        for each_log in data:
            each_log["LOG_TYPE"] = Constant._SYSLOG
            each_log["APPNAME"] =  Constant._THUNDER
            each_log["HOSTNAME"] = self._thunder.get(Constant._RESOURCE_ID) 
            each_log["IP"] = self._thunder.get(Constant._PRIVATE_IP) if Utils.is_valid(self._thunder.get(Constant._PRIVATE_IP)) else self._thunder.get(Constant._IP)
            each_log["AGENT"] = Constant._TOA
            each_log["JOBID"] = Utils._datetime_id
            each_log["PRIORITY"] = Utils.priority(each_log['log-data'])
            each_log["PARTITION"] = self._thunder.get(Constant._PARTITION).upper()
            yield each_log
    
    def log(self, data):
        if(Utils.is_valid(data) and len(data)>0):
            self._logger.info(Constant._SUCCESS_METRIC.format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper(), len(data), data))
            return Constant._SUCCESS 
        return None
    
    def collect_thunders_from_autoscale(self):
        
        headers = {Constant._AUTHORIZATION: "Bearer " + self._token}
        response = Utils.rest_api(Constant._AZURE_VIRTUALMACHINES_URL.format(self._thunder.get(Constant._RESOURCE_ID)), headers, {}, Constant._GET, Constant._ERROR_AZURE_LOGS, self._thunder.get(Constant._IP))
        vmss_obj = response.json()
        public_ip_list = []

        def get_azure_public_ip(vm):
            if(Utils.is_valid(vm.get("properties").get("networkProfile").get("networkInterfaces"), vm.get("properties").get("networkProfile").get("networkInterfaces")[0] )):
                interface_id = vm.get("properties").get("networkProfile").get("networkInterfaces")[0].get("id")
                response = Utils.rest_api(Constant._AZURE_BASE_URL.format(interface_id), headers, {}, Constant._GET, Constant._ERROR_AZURE_AUTOSCALE_THUNDERS, self._thunder.get(Constant._RESOURCE_ID))
                interface_obj = response.json()
                public_ip_id = interface_obj.get("properties").get("ipConfigurations")[0].get(
                    "properties").get("publicIPAddress").get("id")
                response = Utils.rest_api(Constant._AZURE_BASE_URL.format(public_ip_id), headers, {}, Constant._GET, Constant._ERROR_AZURE_AUTOSCALE_THUNDERS, self._thunder.get(Constant._RESOURCE_ID))
                public_obj = response.json()
                public_ip = public_obj.get("properties").get("ipAddress")
                return {
                          Constant._IP: public_ip, 
                          Constant._USERNAME: self._thunder.get(Constant._USERNAME, "admin"),
                          Constant._PASSWORD: self._thunder.get(Constant._PASSWORD), 
                          Constant._RESOURCE_ID: vm.get("id"),
                          Constant._PARTITION: self._thunder.get(Constant._PARTITION, "SHARED")
                    } 
            return None
           
        if vmss_obj.get("value"):
            with ThreadPoolExecutor(int(Utils._main.get(Constant._THREADPOOL_MAX_WORKERS))) as executor:
                futures_vm = {executor.submit(get_azure_public_ip, vm): vm for vm in
                              vmss_obj.get("value")}
                for future in concurrent.futures.as_completed(futures_vm):
                    if future.result() is not None:
                        public_ip_list.append(future.result())
                if(len(public_ip_list)>0):
                    return public_ip_list
                else:
                    return None
        else:
            return None


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT

"""
Configure logging framework and logs asynchronously thread processes.
To use, simply 'import logging' and log away!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

import logging.config
from common.toa_constant import Constant 

class Logging:

    __instance__ = None 
    
    @staticmethod
    def get_logger(className):
        if Logging.__instance__ is None:
            for name in ['boto', 'urllib3', 's3transfer', 'boto3', 'botocore', 'nose']:
                logging.getLogger(name).setLevel(logging.CRITICAL)
            logging.config.fileConfig(fname=Constant._LOGGING_CONF_PATH)
            Logging.__instance__ = logging
        return Logging.__instance__.getLogger(className)


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Data collection from thunder devices.
To use, simply 'import ThunderHandler' and use it!
"""
import asyncio
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from common.toa_constant import Constant
from common.toa_utils import Utils
import json
import datetime
from common.toa_logging import Logging
import socket

class ThunderHandler:
    _logger = Logging().get_logger("ThunderHandler")
    _metric=None
    _date=None
    _thunder=None
    _token=None
    _url=None
    _headers=None
    _temp=None
    _callback=None
      
    def __init__(self, metric, date, thunder, token, callback):
        self._metric=metric
        self._thunder=thunder
        self._date=date
        self._url=Constant._THUNDER_BASE_URL.format(self._thunder.get(Constant._IP))
        self._token = token
        self._headers= Utils.get_headers(token)
        self._callback=callback
    
    def signout(self):
        try:
            return asyncio.run(self.async_logout())
        except Exception:
            return self.sync_logout()
    
    async def async_logout(self):
        response = Utils.rest_api(self._url +"/logoff" , Utils.get_headers(self._token), {}, Constant._GET, Constant._ERROR_THUNDER_LOGOFF_RESPONSE, self._thunder.get(Constant._IP))
        if(Utils.is_valid(response)):
            return Constant._SUCCESS
        return None 
    
    def sync_logout(self):
        response = Utils.rest_api(self._url +"/logoff" , Utils.get_headers(self._token), {}, Constant._GET, Constant._ERROR_THUNDER_LOGOFF_RESPONSE, self._thunder.get(Constant._IP))
        if(Utils.is_valid(response)):
            return Constant._SUCCESS
        return None 
    
    
    def read(self):
        self._temp = Utils.load_temp_storage(self._thunder.get(Constant._IP) + "-" + self._thunder.get(Constant._PARTITION))
    
    def token(self):
       
        _body = json.dumps({
            Constant._CREDENTIALS: {
                Constant._USERNAME: self._thunder.get(Constant._USERNAME),
                Constant._PASSWORD: self._thunder.get(Constant._PASSWORD),
            }
        })
        _headers = {
            Constant._CONTENT_TYPE: Constant._APPLICATION_JSON
        }
        response = Utils.rest_api(self._url +"/auth" , _headers, _body, Constant._POST, Constant._ERROR_THUNDER_AUTH_RESPONSE, self._thunder.get(Constant._IP))
        if(Utils.is_valid(response)):
            self._token = f"A10 {response.json()['authresponse']['signature']}"
            self.internal()
            self.header()
        return self._token
    
    def dates(self):
        response = Utils.rest_api(self._url+"/clock/show/oper", self._headers, {}, Constant._GET, Constant._ERROR_THUNDER_CLOCK_NOT_FOUND, self._thunder.get(Constant._IP))
        thunder = json.loads(response.text)
        time = thunder.get("show").get("oper").get("time")
        date = thunder.get("show").get("oper").get("date")
        date_time_obj = datetime.datetime.strptime(date + time, "%b %d %Y%H:%M:%S")
        date_time_obj = date_time_obj.replace(second=0, microsecond=0)
        
        log_delay = int(Utils._main.get(Constant._LOG_DELAY_MIN))*60
        cron_delay= int(Utils._main.get(Constant._CRON_DELAY_MIN))*60
        start_delta = log_delay + cron_delay 
        
        log_start_time = date_time_obj - datetime.timedelta(seconds=int(start_delta))
        log_end_time = log_start_time + datetime.timedelta(seconds=int(cron_delay))
        
        mins = (log_end_time - log_start_time).total_seconds()/60
        times = []
        for i in range(0, int(mins), 1):
            next_time_delta=i*60
            fetch_time = log_start_time + datetime.timedelta(seconds=int(next_time_delta))
            times.append(fetch_time.strftime("%b %d %Y %H:%M"))
            
        return times
        
    def collect_log(self):
        
        response = Utils.rest_api(self._url+"/syslog/oper?log-data-search={}".format(self._date), Utils.get_headers(self._token), {}, Constant._GET, Constant._ERROR_THUNDER_CLOCK_NOT_FOUND, self._thunder.get(Constant._IP))
        if(Utils.is_valid(response.text) and response.status_code==200):
            response_data = json.loads(response.text)
            oper = response_data["syslog"]["oper"]
            return oper["system-log"]
        return None
    
    def header(self):
        self._headers = Utils.get_headers(self._token)
    
    def collect_metric_cps(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/system/app-performance/stats", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_CPS_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                total_new_connection_per_sec = response.json()['app-performance']['stats']['total-new-conns-per-sec']
                result = {"Total New Connection (Sec)": total_new_connection_per_sec}
        
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None
            
    def collect_metric_cpu(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/system-cpu/data-cpu/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_CPU_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:    
                cpu_usage_list = response.json()['data-cpu']['oper']['cpu-usage']
                total_cpu_usage = 0
                for cpu_usage in cpu_usage_list:
                    total_cpu_usage += cpu_usage['60-sec']
                result = {"CPU Usage Percentage (Data)": total_cpu_usage / len(cpu_usage_list)}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None
            
    def collect_metric_memory(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/system/memory/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_MEMORY_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                memory_usage = response.json()['memory']['oper']['Usage']
                result = {"Memory Usage Percentage": float(memory_usage.rstrip("%"))}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None
            
    def collect_metric_disk(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/system/hardware/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_DISK_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                disk_usage = response.json()['hardware']['oper']['disk-percentage']
                result =  {"Disk Usage Percentage": disk_usage}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None
            
    def collect_metric_throughput(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/system/throughput/stats", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_THROUGHPUT_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                global_throughput = response.json()['throughput']['stats']['global-system-throughput-bits-per-sec']
                result =  {"Throughput Rate (Global/BPS)": global_throughput}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None

    def collect_metric_interface(self): 
        result = None
        try:
            count = 0
            response = Utils.rest_api(self._url+"/interface/available-eth-list/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_INTERFACES_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                json_data = response.json()
                interfaces_list = json_data["available-eth-list"]['oper'].get("if-list", [])
                if(interfaces_list is None):
                    return {"Interface Down Count (Data)": count}
                for i in interfaces_list:
                    state = (i['state'])
                    if state == 'UP':
                        pass
                    else:
                        count += 1
                result = {"Interface Down Count (Data)": count}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None

    def collect_metric_transaction(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/system/app-performance/stats", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_TPS_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                transaction_per_sec = response.json()['app-performance']['stats']['l7-trans-per-sec']
                result = {"Transactions Rate (Sec)": transaction_per_sec}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None

    def collect_metric_server_down(self): 
        result = None
        try:
            count=0
            response = Utils.rest_api(self._url+"/slb/server", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_SERVER_DOWN_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                if(Utils.is_valid(response.text) and Utils.is_valid(response.json())):
                    server_list = response.json().get("server-list", [])
                    if server_list is None or server_list.__len__().__eq__(0):
                        return {"Server Down Count": count}
                    for server in server_list:
                        server_name = server['name']
                        response = Utils.rest_api(self._url+"/slb/server/"+server_name+"/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_SERVER_DOWN_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
                        server_down_status = response.json()['server']['oper']['state']
                        if server_down_status == "Down":
                            count += 1
                result =  {"Server Down Count": count}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None

    def collect_metric_server_down_per(self): 
        result = None
        try:
            server_down_count = 0
            total_server = 0
            server_down_percentage = 0
            response = Utils.rest_api(self._url+"/slb/server", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_SERVER_PER_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                if(Utils.is_valid(response.text) and Utils.is_valid(response.json())):
                    server_list = response.json().get("server-list", [])
                    if server_list is None or server_list.__len__().__eq__(0):
                        return {"Server Down Percentage": 0}
                    for server in server_list:
                        server_name = server['name']
                        response = Utils.rest_api(self._url+"/slb/server/"+server_name+"/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_SERVER_DOWN_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
                        total_server += 1
                        server_down_status = response.json()['server']['oper']['state']
                        if server_down_status == "Down":
                            server_down_count += 1
                    server_down_percentage = ((server_down_count / total_server) * 100)
                result =  {"Server Down Percentage": server_down_percentage}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None

    def collect_metric_ssl(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/slb/ssl-stats/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_SSL_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                ssl_errors_count = response.json()['ssl-stats']['oper']['server-cert-errors']
                result = {"SSL Errors Count": ssl_errors_count}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None


    def collect_metric_server_error(self): 
        result = None
        try:
            total_error_count = 0
            response = Utils.rest_api(self._url+"/slb/http-proxy/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_SERVER_ERROR_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                total_proxy_list = response.json()['http-proxy']['oper']['http-proxy-cpu-list']
                for error_count in total_proxy_list:
                    total_error_count += error_count.get('response_4xx', 0)
                    total_error_count +=error_count.get('response_5xx', 0)
                result = {"Server Errors Count": total_error_count}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None


    def collect_metric_session(self): 
        result = None
        try:
            response = Utils.rest_api(self._url+"/system/session/stats", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_SESSION_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                total_session = response.json()['session']['stats']['total_curr_conn']
                result =  {"Total Session Count": total_session}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None


    def collect_metric_packet_rate(self): 
        result = None
        try:
            self.read()
            old_timestamp_formatted = self._temp.get(Constant._TEMP_LAST_PACKET_SENT_TIME, "")
            current_timestamp = datetime.datetime.utcnow().strftime("%Y-%m-%d, %H:%M:%S")
            current_timestamp_formatted = datetime.datetime.strptime(current_timestamp, "%Y-%m-%d, %H:%M:%S")
            last_packet_count = self._temp.get(Constant._TEMP_LAST_PACKET_COUNT, 0)
            response = Utils.rest_api(self._url+"/plat-cpu-packet/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_PACKET_RATE_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if(not Utils.is_valid(response)):
                result = None
            else:
                all_cpus=None
                if(Utils.is_valid(response)):
                    all_cpus = response.json()['plat-cpu-packet']['oper']['pkt-stats']
                
                current_packet_count = 0
                if(Utils.is_valid(all_cpus)):
                    for cpu in all_cpus:
                        current_packet_count += cpu["pkt-sent"]
    
                if old_timestamp_formatted != "":
                    old_timestamp = datetime.datetime.strptime(old_timestamp_formatted.replace(",", ", "), "%Y-%m-%d, %H:%M:%S")
                    time_interval = (current_timestamp_formatted - old_timestamp).total_seconds()
                    # packet get sent in last minute
                    if current_packet_count > last_packet_count:
                        packet_rate = (current_packet_count - last_packet_count) / time_interval
                        self._temp[Constant._TEMP_LAST_PACKET_SENT_TIME] = current_timestamp
                        self._temp[Constant._TEMP_LAST_PACKET_COUNT] = current_packet_count
    
                    # no packet get sent in last minute
                    elif current_packet_count == last_packet_count:
                        packet_rate = 0
                        self._temp[Constant._TEMP_LAST_PACKET_SENT_TIME] = current_timestamp
                    # vThunder reboot
                    else:
                        packet_rate = 0
                        self._temp[Constant._TEMP_LAST_PACKET_SENT_TIME] = current_timestamp
                        self._temp[Constant._TEMP_LAST_PACKET_COUNT] = current_packet_count
                # first time crone job run
                else:
                    packet_rate = 0
                    self._temp[Constant._TEMP_LAST_PACKET_SENT_TIME] = current_timestamp
                    self._temp[Constant._TEMP_LAST_PACKET_COUNT] = current_packet_count
    
                Utils.write(self._temp, self._thunder.get(Constant._IP)+"-"+self._thunder.get(Constant._PARTITION))
                result =  {"Packet Rate (Sec)": int(packet_rate)}
            return self.callback(result)
        except Exception as exp:
            self._logger.debug(exp)
            return None
        
    def collect_metric_packet_drop(self): 
            result = None
            try:
                self.read()
                old_timestamp = self._temp.get(Constant._TEMP_LAST_PACKET_DROPPED_SENT_TIME, "")
                current_timestamp1 = datetime.datetime.utcnow().strftime("%Y-%m-%d, %H:%M:%S")
                current_timestamp = datetime.datetime.strptime(current_timestamp1, "%Y-%m-%d, %H:%M:%S")
                
                response = Utils.rest_api(self._url+"/plat-cpu-packet/oper", self._headers, {}, Constant._GET, Constant._ERROR_METRIC_PACKET_DROP_NOT_FOUND, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
                if(not Utils.is_valid(response)):
                    result = None
                else:
                    all_cpus=None
                    last_packet_dropped = 0
                    if(Utils.is_valid(response)):
                        last_packet_dropped = self._temp.get(Constant._TEMP_LAST_PACKET_DROPPED_COUNT, 0)
                        all_cpus = response.json()['plat-cpu-packet']['oper']['pkt-stats']
                    
                    current_packet_dropped = 0
                    if(Utils.is_valid(all_cpus)):
                        for cpu in all_cpus:
                            current_packet_dropped += cpu["pkt-drop"]
        
                    if old_timestamp != "":
                        old_timestamp = old_timestamp.replace(",", ", ")
                        old_timestamp = datetime.datetime.strptime(old_timestamp,
                                                                   "%Y-%m-%d, %H:%M:%S")
                        time_interval = (current_timestamp - old_timestamp).total_seconds()
                        # packet get dropped in last minute
                        if current_packet_dropped > last_packet_dropped:
                            packet_dropped = (current_packet_dropped -
                                              last_packet_dropped) / time_interval
                            self._temp[Constant._TEMP_LAST_PACKET_DROPPED_SENT_TIME] = current_timestamp1
                            self._temp[Constant._TEMP_LAST_PACKET_DROPPED_COUNT] = current_packet_dropped
        
                        # no packet get dropped in last minute
                        elif current_packet_dropped == last_packet_dropped:
                            packet_dropped = 0
                            self._temp[Constant._TEMP_LAST_PACKET_DROPPED_SENT_TIME] = current_timestamp1
                        # vThunder reboot
                        else:
                            packet_dropped = 0
                            self._temp[Constant._TEMP_LAST_PACKET_DROPPED_SENT_TIME] = current_timestamp1
                            self._temp[Constant._TEMP_LAST_PACKET_DROPPED_COUNT] = current_packet_dropped
                    # first time crone job run
                    else:
                        packet_dropped = 0
                        self._temp[Constant._TEMP_LAST_PACKET_DROPPED_SENT_TIME] = current_timestamp1
                        self._temp[Constant._TEMP_LAST_PACKET_DROPPED_COUNT] = current_packet_dropped
        
                    Utils.write(self._temp, self._thunder.get(Constant._IP)+"-"+self._thunder.get(Constant._PARTITION))
                    result = {"Packet Drop Rate (Sec)": int(packet_dropped)}
                return self.callback(result)
         
            except Exception as exp:
                self._logger.debug(exp)
                return None
                                               
    def collect_metric(self):
        
        if(self._metric==Constant._AWS_CPS or self._metric==Constant._AZURE_CPS or self._metric==Constant._VMWARE_CPS):
            return self.collect_metric_cps()
        
        elif(self._metric==Constant._AWS_CPU or self._metric==Constant._AZURE_CPU or self._metric==Constant._VMWARE_CPU): 
            return self.collect_metric_cpu()
            
        elif(self._metric==Constant._AWS_MEMORY or self._metric==Constant._AZURE_MEMORY or self._metric==Constant._VMWARE_MEMORY):
            return self.collect_metric_memory()
        
        elif(self._metric==Constant._AWS_DISK or self._metric==Constant._AZURE_DISK or self._metric==Constant._VMWARE_DISK):
            return self.collect_metric_disk()
        
        elif(self._metric==Constant._AWS_THROUGHPUT or self._metric==Constant._AZURE_THROUGHPUT or self._metric==Constant._VMWARE_THROUGHPUT):
            return self.collect_metric_throughput()
        
        elif(self._metric==Constant._AWS_INTERFACES or self._metric==Constant._AZURE_INTERFACES or self._metric==Constant._VMWARE_INTERFACES):
            return self.collect_metric_interface()
        
        elif(self._metric==Constant._AWS_TPS or self._metric==Constant._AZURE_TPS or self._metric==Constant._VMWARE_TPS):
            return self.collect_metric_transaction()
        
        elif(self._metric==Constant._AWS_SERVER_DOWN_COUNT or self._metric==Constant._AZURE_SERVER_DOWN_COUNT or self._metric==Constant._VMWARE_SERVER_DOWN_COUNT):
            return self.collect_metric_server_down()
            
        elif(self._metric==Constant._AWS_SERVER_DOWN_PERCENTAGE or self._metric==Constant._AZURE_SERVER_DOWN_PERCENTAGE or self._metric==Constant._VMWARE_SERVER_DOWN_PERCENTAGE):
            return self.collect_metric_server_down_per()

        elif(self._metric==Constant._AWS_SSL_CERT or self._metric==Constant._AZURE_SSL_CERT  or self._metric==Constant._VMWARE_SSL_CERT ):
            return self.collect_metric_ssl()
        
        elif(self._metric==Constant._AWS_SERVER_ERROR or self._metric==Constant._AZURE_SERVER_ERROR or self._metric==Constant._VMWARE_SERVER_ERROR):
            return self.collect_metric_server_error()
            
        elif(self._metric==Constant._AWS_SESSION or self._metric==Constant._AZURE_SESSION or self._metric==Constant._VMWARE_SESSION):
            return self.collect_metric_session()
            
        elif(self._metric==Constant._AWS_PACKET_RATE or self._metric==Constant._AZURE_PACKET_RATE or self._metric==Constant._VMWARE_PACKET_RATE):
            return self.collect_metric_packet_rate()
            
        elif(self._metric==Constant._AWS_PACKET_DROP or self._metric==Constant._AZURE_PACKET_DROP or self._metric==Constant._VMWARE_PACKET_DROP):
            return self.collect_metric_packet_drop()

    def partitions(self):
        try:
            response = Utils.rest_api(self._url+"/partition", self._headers, {}, Constant._GET, Constant._ERROR_PARTITION_NOT_FOUND, self._thunder.get(Constant._IP))
            if(not Utils.is_valid(response)): 
                return None
            else:
                if(Utils.is_valid(response.text) and Utils.is_valid(response.json()['partition-list'])):
                    partitions = response.json()['partition-list']
                    return [partition.get(Constant._PARTITION_NAME, None) for partition in partitions] 
        except Exception as exp:
            self._logger.debug(exp)
        return None
        
    def activate(self):
        if(self._thunder.get(Constant._PARTITION)==Constant._SHARED):
            body = json.dumps({ "active-partition": { "shared": 1}})
        else:
            body = json.dumps({ "active-partition": { "curr_part_name": self._thunder.get(Constant._PARTITION)}})
        
        try:
            response = Utils.rest_api(self._url+"/active-partition", self._headers, body, Constant._POST, Constant._ERROR_PARTITION, self._thunder.get(Constant._IP))
            if(not Utils.is_valid(response)): 
                return None
            else:
                return Constant._SUCCESS
        except Exception as exp:
            self._logger.debug(exp)
        return None
    
    def activemode(self):
        try:
            response = Utils.rest_api(self._url+"/vrrp-a/vrid/oper", self._headers, {}, Constant._GET, Constant._ERROR_PARTITION_NOT_FOUND, self._thunder.get(Constant._IP))
            if(not Utils.is_valid(response)): 
                return True
            else:
                if(Utils.is_valid(response.text) and Utils.is_valid(response.json()['vrid-list']) and Utils.is_valid(response.json()['vrid-list'][0]) and Utils.is_valid(response.json()['vrid-list'][0]['oper'])):
                    state = response.json()['vrid-list'][0]['oper']['state']
                    if(state == "vrrp-a not enabled"):
                        return True
                    else:
                        return response.json()['vrid-list'][0]['oper']['state']=='Active' 
        except Exception as exp:
            self._logger.debug(exp)
        return None
    
        

    def callback(self, result):
        if(Utils.is_valid(self._callback)):
            return self._callback(result)
        else:
            return result
    
    def internal(self):
        if(self._thunder.get(Constant._IP) =="127.0.0.1"):
            response = Utils.rest_api(self._url +"/interface/management/oper" , Utils.get_headers(self._token), {}, Constant._GET, Constant._ERROR_THUNDER_MANG_IP_NOT_FOUND, self._thunder.get(Constant._IP))
            if(Utils.is_valid(response)):
                json_responce = json.loads(response.text).get("management") 
                self._thunder[Constant._PRIVATE_IP] =  json_responce.get("oper", {}).get("ipv4-addr")
                if(Utils._active_log_provider == Constant._AZURE_LOG or Utils._active_metric_provider == Constant._VMWARE_METRIC):
                    pass
                else:
                    self._thunder[Constant._RESOURCE_ID] = socket.gethostname()    
                    
            else:
                self._thunder[Constant._PRIVATE_IP] = None
        else:
            self._thunder[Constant._PRIVATE_IP] = None                


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Processing engine to process partition information from thunder to configured provider.
To use, simply 'import PartitionProcessor' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from common.toa_constant import Constant
from common.toa_utils import Utils
from common.toa_logging import Logging
from handler.thunder_handler import ThunderHandler
from concurrent.futures import ThreadPoolExecutor
from processor.log_processor import LogProcessor
from processor.metric_processor import MetricProcessor

class PartitionProcessor:
    
    _logger = Logging().get_logger("PartitionProcessor") 
    
    def process(self, thunder):
        handler = ThunderHandler(None, None, thunder, None, callback=None) 
        try:
            handler.token()
            if(not Utils.is_valid(handler._token)):
                return None
            
            thunders = [handler]
           
            with ThreadPoolExecutor(max_workers=int(Utils._main.get(Constant._THREADPOOL_MAX_WORKERS))) as executor:
                if(not handler._thunder.get(Constant._NAMESPACE)==Constant._THUNDER):
                    handler.activate()
                    if(handler.activemode()): 
                        executor.map(LogProcessor().process, thunders)
                        executor.map(MetricProcessor(metric=Utils._metrics, partition=Constant._L3V).process, thunders)
                    else:
                        Utils._logger.warn((Constant._WARN_STANDBY_PARTITION).format(handler._thunder.get(Constant._PARTITION), handler._thunder.get(Constant._IP)+"-"+handler._thunder.get(Constant._NAMESPACE))) 
    
                elif(handler._thunder.get(Constant._NAMESPACE)==Constant._THUNDER):
                    executor.map(MetricProcessor(metric=Utils._non_l4v_metrics, partition=Constant._NON_L3V).process, thunders) 
                    
        except Exception as exp:
            self._logger.debug(exp)
        handler.signout()
        


#!/usr/bin/python3
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Configure and run thunder agent.
To use, simply 'import Toa' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from common.toa_helper import Helper
from common.toa_logging import Logging
from common.toa_utils import Utils
from common.toa_constant import Constant

_logger = Logging().get_logger("main")

if __name__ == "__main__":
    _job_start_time = Utils._datetime
    _logger.info(Constant._INFO_JOB_ID.format(str(Utils._datetime_id)))
    _logger.info(Constant._INFO_JOB_START.format(str(_job_start_time)))

    Helper.init()

    _job_end_time=Utils.date_time_utc()
    _logger.info(Constant._INFO_JOB_TIME.format(( _job_end_time - _job_start_time).total_seconds()))
    _logger.info(Constant._INFO_JOB_END.format(str(_job_end_time)))
    _logger.info(Constant._DOCS)
    _logger.info("##### TOA  ###### All Rights Reserved @A10 Networks Inc ##### TOA #####")
    
    



#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Processing engine to process metrics information from thunder to configured provider.
To use, simply 'import Metric' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from common.toa_constant import Constant
from common.toa_utils import Utils
from concurrent.futures import ThreadPoolExecutor
from common.toa_logging import Logging
from handler.thunder_handler import ThunderHandler
from handler.aws_handler import AWSHandler
from handler.vmware_handler import VMWareHandler
from handler.azure_handler import AzureHandler
import concurrent.futures
class LogProcessor:
    
    _logger = Logging().get_logger("LogProcessor")
    
    def process(self, thunder):
        if(Utils.is_valid(Utils._active_log_provider)):
            if Utils._active_log_provider==Constant._AWS_LOG:
                self.process_aws_log(thunder)
            elif Utils._active_log_provider==Constant._AZURE_LOG:
                self.process_azure_log(thunder)
            elif Utils._active_log_provider==Constant._VMWARE_LOG:
                self.process_vmware_log(thunder)
            
    def process_aws_log(self, thunder):
        logs = self.collect_log(thunder)
        if(Utils.is_valid(logs)):
            handler = AWSHandler(thunder._thunder, Constant._AWS_BOTO3_SERVICE_LOGS)  
            handler.publish_log(logs)
       
    def process_vmware_log(self, thunder): 
        logs = self.collect_log(thunder)
        if(Utils.is_valid(logs)):
            handler = VMWareHandler(thunder._thunder, None)
            handler.publish_log(logs) 
       
    def process_azure_log(self, thunder): 
        logs = self.collect_log(thunder)
        if(Utils.is_valid(logs)):
            handler = AzureHandler(thunder._thunder, None)
            handler.publish_log(logs)
    
    def collect_log(self, thunder): 
        logs_data = []
        dates = thunder.dates()
        
        thunders = [ThunderHandler(metric=None, date=date, thunder=thunder._thunder, token=thunder._token, callback=None) for date in dates]
        with ThreadPoolExecutor(max_workers=int(Utils._main.get(Constant._THREADPOOL_MAX_WORKERS))) as collect:
            futures_logs = [collect.submit(thunder.collect_log) for thunder in thunders]
            for future in concurrent.futures.as_completed(futures_logs):
                if future.result() is not None:
                    logs_data.extend(future.result())
                    
            #asyncio.run(thunder.signout())
            return logs_data
    
    


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT

"""
Configure utility methods for toa framework.
To use, simply 'import Utils' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from common.toa_logging import Logging
import json 
import datetime
from common.toa_constant import Constant
import requests
import asyncio
from requests.exceptions import ConnectTimeout, ReadTimeout

class Utils:
    
    def __init__(self): 
        pass 
    
    _logger = Logging().get_logger("Utils")
    _active_metric_provider=None
    _active_log_provider=None
    _main=None
    _config=None
    _thunders=None
    _aws_credentials=None
    _aws_config=None
    _azure_credentials=None
    _vmware_credentials=None
    _metrics=None
    _non_l4v_metrics=None
    _datetime      = datetime.datetime.now(datetime.timezone.utc)
    _datetime_epoc = int(datetime.datetime.timestamp(_datetime) * 1000)
    _datetime_str  =_datetime.strftime("%Y-%m-%dT%H:%M:%S")
    _datetime_id   =_datetime.strftime("%Y%m%d%H%M%S")
    _datetime_gmt_str =  _datetime.strftime('%a, %d %b %Y %H:%M:%S GMT')
    _http_ssl_verify=None
    
    @staticmethod
    def load_properties():
        Utils._main = Utils.load_configuration(Constant._MAIN_CONF_PATH, Utils._main, Constant._ERROR_MAIN_NOT_FOUND, Constant._KEYVALUE) 
    
    @staticmethod
    def load_config():
        Utils._config = Utils.load_configuration(Utils._main.get(Constant._CONFIG_PATH), Utils._config, Constant._ERROR_CONFIG_NOT_FOUND, Constant._JSON) 
    
    @staticmethod
    def load_collect_provider():
        Utils._thunders = Utils.load_configuration(Utils._main.get(Constant._THUNDER_CREDENTIALS_PATH), Utils._thunders, Constant._ERROR_THUNDER_CREDENTIALS_NOT_FOUND, Constant._JSON)
        
    @staticmethod
    def load_publish_provider():
        Utils.validate_provider()
        Utils.load_active_log_provider()
        Utils.load_active_metric_provider()
        if(not Utils.is_valid(Utils._active_log_provider) and not Utils.is_valid(Utils._active_metric_provider)):
            Utils._logger.warn(Constant._ERROR_NO_VALID_PROVIDER_CONFIG_FOUND)
            
        if(Utils.is_valid(Utils._main)):
            if(not Utils.is_valid(Utils._main.get(Constant._LOG_DELAY_MIN)) or not Utils.is_integer(Utils._main.get(Constant._LOG_DELAY_MIN)) or int(Utils._main.get(Constant._LOG_DELAY_MIN)) < 0):
                Utils._logger.warn((Constant._ERROR_NO_VALID_LOG_DELAY_IN_MIN_FOUND).format(Utils._main.get(Constant._LOG_DELAY_MIN)))
                Utils._main[Constant._LOG_DELAY_MIN] =  0
            
            if(not Utils.is_valid(Utils._main.get(Constant._CRON_DELAY_MIN)) or not Utils.is_integer(Utils._main.get(Constant._CRON_DELAY_MIN)) or int(Utils._main.get(Constant._CRON_DELAY_MIN)) < 0):
                Utils._logger.warn((Constant._ERROR_NO_VALID_CRON_DELAY_IN_MIN_FOUND).format(Utils._main.get(Constant._CRON_DELAY_MIN)))
                Utils._main[Constant._CRON_DELAY_MIN] = 1
                
        if(Utils.is_valid(Utils._active_log_provider) or Utils.is_valid(Utils._active_metric_provider)):
            # Azure validation.
            if(Utils._active_log_provider==Constant._AZURE_LOG or Utils._active_metric_provider==Constant._AZURE_METRIC):
                if(not Utils.is_valid(Utils._azure_credentials)):
                    Utils._azure_credentials =  Utils.load_configuration(Utils._main.get(Constant._AZURE_CREDENTIALS_PATH), Utils._azure_credentials, Constant._ERROR_AZURE_CREDENTIALS_NOT_FOUND, Constant._KEYVALUE) 
                
                if(Utils._active_metric_provider==Constant._AZURE_METRIC):
                    if(not Utils.is_valid(Utils._azure_credentials) or (Utils.is_valid(Utils._azure_credentials) 
                        and (not Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_CLIENT_ID))
                        or not Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_SECRET_ID))
                        or not Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_TENANT_ID))
                        or not Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_LOCATION))))): 
                        Utils._logger.error(Constant._ERROR_AZURE_CREDENTIALS_METRIC_NOT_FOUND)
                        Utils._active_metric_provider=None
                    
                    if(Utils.is_valid(Utils._config)
                        and not Utils.is_valid(Utils._config.get(Constant._AZURE_METRIC_RESOURCE_ID))):  
                        Utils._logger.error(Constant._ERROR_AZURE_CONFIG_METRIC_NOT_FOUND)
                        Utils._active_metric_provider=None
                
                if(Utils._active_log_provider==Constant._AZURE_LOG):
                    if(not Utils.is_valid(Utils._azure_credentials) or (Utils.is_valid(Utils._azure_credentials)
                        and not Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_WORKSPACE_PRIMARY_KEY)))): 
                        Utils._logger.error(Constant._ERROR_AZURE_CREDENTIALS_LOG_NOT_FOUND) 
                        Utils._active_log_provider = None
                                
                    if(Utils.is_valid(Utils._config)
                        and not Utils.is_valid(Utils._config.get(Constant._AZURE_LOG_WORKSPACE_ID))):  
                        Utils._logger.error(Constant._ERROR_AZURE_CONFIG_LOG_NOT_FOUND)
                        Utils._active_log_provider = None
                
            if((Utils._active_log_provider==Constant._AWS_LOG or Utils._active_metric_provider==Constant._AWS_METRIC)):
                if(not Utils.is_valid(Utils._aws_credentials)): 
                    Utils._aws_credentials =  Utils.load_configuration(Utils._main.get(Constant._AWS_CREDENTIALS_PATH), Utils._aws_credentials, Constant._ERROR_AWS_CREDENTIALS_NOT_FOUND, Constant._KEYVALUE) 
                if(not Utils.is_valid(Utils._aws_config)): 
                    Utils._aws_config =  Utils.load_configuration(Utils._main.get(Constant._AWS_CONFIG_PATH), Utils._aws_config, Constant._ERROR_AWS_CONFIG_NOT_FOUND, Constant._KEYVALUE) 
                
                if(not Utils.is_valid(Utils._aws_credentials) or (Utils.is_valid(Utils._aws_credentials)
                    and (not Utils.is_valid(Utils._aws_credentials.get(Constant._AWS_ACCESS_KEY_ID)) 
                    or not Utils.is_valid(Utils._aws_credentials.get(Constant._AWS_SECRET_ACCESS_KEY))))): 
                        Utils._logger.error(Constant._ERROR_AWS_CRED_NOT_FOUND)
                        Utils._active_log_provider=None
                        Utils._active_metric_provider=None
                        
                if(not Utils.is_valid(Utils._aws_config) or (Utils.is_valid(Utils._aws_config)
                    and not Utils.is_valid(Utils._aws_config.get(Constant._AWS_REGION)))):  
                        Utils._logger.error(Constant._ERROR_AWS_CONFIG_METRIC_NOT_FOUND)
                        Utils._active_log_provider=None
                        Utils._active_metric_provider=None
                        
                if(Utils.is_valid(Utils._config) and Utils._active_log_provider==Constant._AWS_LOG 
                        and (not Utils.is_valid(Utils._config.get(Constant._AWS_LOG_GROUP_NAME)))): 
                        Utils._logger.error(Constant._ERROR_AWS_CONFIG_LOG_NOT_FOUND) 
                        Utils._active_log_provider = None
                            
            if(Utils._active_metric_provider==Constant._VMWARE_METRIC or Utils._active_log_provider==Constant._VMWARE_LOG):
                if(not Utils.is_valid(Utils._vmware_credentials)): 
                    Utils._vmware_credentials = Utils.load_configuration(Utils._main.get(Constant._VMWARE_CREDENTIALS_PATH), Utils._vmware_credentials, Constant._ERROR_VMWARE_CREDENTIALS_NOT_FOUND, Constant._KEYVALUE) 
                
                if(not Utils.is_valid(Utils._vmware_credentials) or 
                        (Utils.is_valid(Utils._vmware_credentials) and Utils._active_metric_provider==Constant._VMWARE_METRIC 
                        and (not Utils.is_valid(Utils._vmware_credentials.get(Constant._VMWARE_VROPS_USERNAME)) 
                        or not Utils.is_valid(Utils._vmware_credentials.get(Constant._VMWARE_VROPS_PASSWORD))))):
                        Utils._logger.error(Constant._ERROR_VMWARE_CRED_METRIC_NOT_FOUND)
                        Utils._active_metric_provider = None
                        
                if(Utils.is_valid(Utils._config)
                    and Utils._active_log_provider==Constant._VMWARE_LOG
                    and (not Utils.is_valid(Utils._config.get(Constant._VMWARE_VRLI_HOST)))):
                    Utils._logger.error(Constant._ERROR_VMWARE_CONFIG_LOG_NOT_FOUND)
                    Utils._active_log_provider = None
                    
                if(Utils.is_valid(Utils._config)
                    and Utils._active_metric_provider==Constant._VMWARE_METRIC
                    and (not Utils.is_valid(Utils._config.get(Constant._VMWARE_VROPS_HOST)))):
                    Utils._logger.error(Constant._ERROR_VMWARE_CONFIG_METRIC_NOT_FOUND)
                    Utils._active_metric_provider = None
               
             
    
    @staticmethod
    def validate_provider():
        if(Utils.is_valid(Utils._config)):
            if ((not Utils.is_valid(Utils._config.get(Constant._AZURE_PROVIDER)) or  Utils._config.get(Constant._AZURE_PROVIDER)!=1)
                and ((Utils.is_valid(Utils._config.get(Constant._AZURE_METRIC)) and Utils._config.get(Constant._AZURE_METRIC) == 1)
                or ((Utils.is_valid(Utils._config.get(Constant._AZURE_LOG)) and Utils._config.get(Constant._AZURE_LOG) == 1)))):
                Utils._logger.warn(Constant._ERROR_NO_VALID_PROVIDER_FOUND.format(str(Utils._config.get(Constant._AZURE_METRIC)),str(Utils._config.get(Constant._AZURE_LOG)), Constant._AZURE_PROVIDER))
            
            elif ((not Utils.is_valid(Utils._config.get(Constant._AWS_PROVIDER)) or  Utils._config.get(Constant._AWS_PROVIDER)!=1)
                and ((Utils.is_valid(Utils._config.get(Constant._AWS_METRIC)) and Utils._config.get(Constant._AWS_METRIC) == 1)
                or ((Utils.is_valid(Utils._config.get(Constant._AWS_LOG)) and Utils._config.get(Constant._AWS_LOG) == 1)))):
                Utils._logger.warn(Constant._ERROR_NO_VALID_PROVIDER_FOUND.format(str(Utils._config.get(Constant._AWS_METRIC)),str(Utils._config.get(Constant._AWS_LOG)), Constant._AWS_PROVIDER))
            
            elif ((not Utils.is_valid(Utils._config.get(Constant._VMWARE_PROVIDER)) or  Utils._config.get(Constant._VMWARE_PROVIDER)!=1)
                and ((Utils.is_valid(Utils._config.get(Constant._VMWARE_METRIC)) and Utils._config.get(Constant._VMWARE_METRIC) == 1)
                or ((Utils.is_valid(Utils._config.get(Constant._VMWARE_LOG)) and Utils._config.get(Constant._VMWARE_LOG) == 1)))):
                Utils._logger.warn(Constant._ERROR_NO_VALID_PROVIDER_FOUND.format(str(Utils._config.get(Constant._VMWARE_METRIC)),str(Utils._config.get(Constant._VMWARE_LOG)), Constant._VMWARE_PROVIDER))
            
    @staticmethod
    def load_active_log_provider():
        if(not Utils.is_valid(Utils._config)):
            return None
        if (Utils._config.get(Constant._AZURE_PROVIDER) 
            and Utils._config.get(Constant._AZURE_LOG)
            and Utils._config.get(Constant._AZURE_PROVIDER) == 1
            and Utils._config.get(Constant._AZURE_LOG)==1):
            Utils._logger.info(Constant._INFO_ACTIVE_PROVIDER_LOG.format(Constant._AZURE.upper()))
            Utils._active_log_provider = Constant._AZURE_LOG
        elif (Utils._config.get(Constant._VMWARE_PROVIDER) 
            and Utils._config.get(Constant._VMWARE_LOG)
            and Utils._config.get(Constant._VMWARE_PROVIDER) == 1
            and Utils._config.get(Constant._VMWARE_LOG)==1):
            Utils._logger.info(Constant._INFO_ACTIVE_PROVIDER_LOG.format(Constant._VMWARE.upper()))
            Utils._active_log_provider =  Constant._VMWARE_LOG
        elif (Utils._config.get(Constant._AWS_PROVIDER) 
            and Utils._config.get(Constant._AWS_LOG)
            and Utils._config.get(Constant._AWS_PROVIDER) == 1
            and Utils._config.get(Constant._AWS_LOG)==1):
            Utils._logger.info(Constant._INFO_ACTIVE_PROVIDER_LOG.format(Constant._AWS.upper()))
            Utils._active_log_provider =  Constant._AWS_LOG
        if(Utils.is_valid(Utils._active_log_provider)):
            Utils._logger.info(Constant._INFO_ACTIVE_LOG_PROVIDER_FOUND.format(Utils._active_log_provider.upper())) 
        count = 0    
        
        if(Utils._config.get(Constant._AZURE_LOG)==1):
            count = count + 1
        if(Utils._config.get(Constant._VMWARE_LOG)==1):
            count = count + 1    
        if(Utils._config.get(Constant._AWS_LOG)==1):
            count = count + 1
            
        if(count > 1):
            Utils._logger.warn(Constant._WARN_MULTIPLE_LOG_FOUND) 
                   
    @staticmethod
    def load_active_metric_provider():
        if(not Utils.is_valid(Utils._config)):
            return None
        if (Utils._config.get(Constant._AZURE_PROVIDER) 
            and Utils._config.get(Constant._AZURE_METRIC) 
            and Utils._config.get(Constant._AZURE_PROVIDER) == 1
            and Utils._config.get(Constant._AZURE_METRIC)==1):
            Utils._logger.info(Constant._INFO_ACTIVE_PROVIDER_METRIC.format(Constant._AZURE.upper()))
            Utils._active_metric_provider=Constant._AZURE_METRIC
            Utils._metrics = []
            Utils._non_l4v_metrics = []
            if(Utils._config.get(Constant._AZURE_CPU)==1):
                Utils._metrics.append(Constant._AZURE_CPU)
                
            if(Utils._config.get(Constant._AZURE_MEMORY)==1):
                Utils._non_l4v_metrics.append(Constant._AZURE_MEMORY)
                
            if(Utils._config.get(Constant._AZURE_DISK)==1):
                Utils._non_l4v_metrics.append(Constant._AZURE_DISK)
                
            if(Utils._config.get(Constant._AZURE_THROUGHPUT)==1):
                Utils._metrics.append(Constant._AZURE_THROUGHPUT)
                
            if(Utils._config.get(Constant._AZURE_INTERFACES)==1):
                Utils._metrics.append(Constant._AZURE_INTERFACES)
                
            if(Utils._config.get(Constant._AZURE_CPS)==1):
                Utils._metrics.append(Constant._AZURE_CPS)
                
            if(Utils._config.get(Constant._AZURE_TPS)==1):
                Utils._metrics.append(Constant._AZURE_TPS)
                
            if(Utils._config.get(Constant._AZURE_SERVER_DOWN_COUNT)==1):
                Utils._metrics.append(Constant._AZURE_SERVER_DOWN_COUNT) 
                
            if(Utils._config.get(Constant._AZURE_SERVER_DOWN_PERCENTAGE)==1):
                Utils._metrics.append(Constant._AZURE_SERVER_DOWN_PERCENTAGE)
                
            if(Utils._config.get(Constant._AZURE_SSL_CERT)==1):
                Utils._metrics.append(Constant._AZURE_SSL_CERT) 
               
            if(Utils._config.get(Constant._AZURE_SERVER_ERROR)==1):
                Utils._metrics.append(Constant._AZURE_SERVER_ERROR)
                
            if(Utils._config.get(Constant._AZURE_SESSION)==1):
                Utils._metrics.append(Constant._AZURE_SESSION)
                
            if(Utils._config.get(Constant._AZURE_PACKET_RATE)==1):
                Utils._metrics.append(Constant._AZURE_PACKET_RATE)
                
            if(Utils._config.get(Constant._AZURE_PACKET_DROP)==1):
                Utils._metrics.append(Constant._AZURE_PACKET_DROP)
            
        elif (Utils._config.get(Constant._VMWARE_PROVIDER) 
            and Utils._config.get(Constant._VMWARE_METRIC)
            and Utils._config.get(Constant._VMWARE_PROVIDER) == 1
            and Utils._config.get(Constant._VMWARE_METRIC)==1):
            Utils._active_metric_provider=Constant._VMWARE_METRIC;
            Utils._logger.info(Constant._INFO_ACTIVE_PROVIDER_METRIC.format(Constant._VMWARE.upper()))
            Utils._metrics = []
            Utils._non_l4v_metrics = []
            if(Utils._config.get(Constant._VMWARE_CPU)==1):
                Utils._metrics.append(Constant._VMWARE_CPU)
                
            if(Utils._config.get(Constant._VMWARE_MEMORY)==1):
                Utils._non_l4v_metrics.append(Constant._VMWARE_MEMORY)
                
            if(Utils._config.get(Constant._VMWARE_DISK)==1):
                Utils._non_l4v_metrics.append(Constant._VMWARE_DISK)
                
            if(Utils._config.get(Constant._VMWARE_THROUGHPUT)==1):
                Utils._metrics.append(Constant._VMWARE_THROUGHPUT)
                
            if(Utils._config.get(Constant._VMWARE_INTERFACES)==1):
                Utils._metrics.append(Constant._VMWARE_INTERFACES)
                
            if(Utils._config.get(Constant._VMWARE_CPS)==1):
                Utils._metrics.append(Constant._VMWARE_CPS)
                
            if(Utils._config.get(Constant._VMWARE_TPS)==1):
                Utils._metrics.append(Constant._VMWARE_TPS)
                
            if(Utils._config.get(Constant._VMWARE_SERVER_DOWN_COUNT)==1):
                Utils._metrics.append(Constant._VMWARE_SERVER_DOWN_COUNT) 
                
            if(Utils._config.get(Constant._VMWARE_SERVER_DOWN_PERCENTAGE)==1):
                Utils._metrics.append(Constant._VMWARE_SERVER_DOWN_PERCENTAGE)
                
            if(Utils._config.get(Constant._VMWARE_SSL_CERT)==1):
                Utils._metrics.append(Constant._VMWARE_SSL_CERT) 
               
            if(Utils._config.get(Constant._VMWARE_SERVER_ERROR)==1):
                Utils._metrics.append(Constant._VMWARE_SERVER_ERROR)
                
            if(Utils._config.get(Constant._VMWARE_SESSION)==1):
                Utils._metrics.append(Constant._VMWARE_SESSION)
                
            if(Utils._config.get(Constant._VMWARE_PACKET_RATE)==1):
                Utils._metrics.append(Constant._VMWARE_PACKET_RATE)
                
            if(Utils._config.get(Constant._VMWARE_PACKET_DROP)==1):
                Utils._metrics.append(Constant._VMWARE_PACKET_DROP)
                
        elif (Utils._config.get(Constant._AWS_PROVIDER) 
            and Utils._config.get(Constant._AWS_METRIC)
            and Utils._config.get(Constant._AWS_PROVIDER) == 1
            and Utils._config.get(Constant._AWS_METRIC)==1):
            Utils._active_metric_provider=Constant._AWS_METRIC;
            Utils._logger.info(Constant._INFO_ACTIVE_PROVIDER_METRIC.format(Constant._AWS.upper()))
            Utils._metrics = []
            Utils._non_l4v_metrics = []
            if(Utils._config.get(Constant._AWS_CPU)==1):
                Utils._metrics.append(Constant._AWS_CPU)
                
            if(Utils._config.get(Constant._AWS_MEMORY)==1):
                Utils._non_l4v_metrics.append(Constant._AWS_MEMORY)
                
            if(Utils._config.get(Constant._AWS_DISK)==1):
                Utils._non_l4v_metrics.append(Constant._AWS_DISK)
                
            if(Utils._config.get(Constant._AWS_THROUGHPUT)==1): 
                Utils._metrics.append(Constant._AWS_THROUGHPUT)
                
            if(Utils._config.get(Constant._AWS_INTERFACES)==1):
                Utils._metrics.append(Constant._AWS_INTERFACES)
                
            if(Utils._config.get(Constant._AWS_CPS)==1):
                Utils._metrics.append(Constant._AWS_CPS)
                
            if(Utils._config.get(Constant._AWS_TPS)==1):
                Utils._metrics.append(Constant._AWS_TPS)
                
            if(Utils._config.get(Constant._AWS_SERVER_DOWN_COUNT)==1):
                Utils._metrics.append(Constant._AWS_SERVER_DOWN_COUNT) 
                
            if(Utils._config.get(Constant._AWS_SERVER_DOWN_PERCENTAGE)==1):
                Utils._metrics.append(Constant._AWS_SERVER_DOWN_PERCENTAGE)
                
            if(Utils._config.get(Constant._AWS_SSL_CERT)==1):
                Utils._metrics.append(Constant._AWS_SSL_CERT) 
               
            if(Utils._config.get(Constant._AWS_SERVER_ERROR)==1):
                Utils._metrics.append(Constant._AWS_SERVER_ERROR)
                
            if(Utils._config.get(Constant._AWS_SESSION)==1):
                Utils._metrics.append(Constant._AWS_SESSION)
                
            if(Utils._config.get(Constant._AWS_PACKET_RATE)==1):
                Utils._metrics.append(Constant._AWS_PACKET_RATE)
                
            if(Utils._config.get(Constant._AWS_PACKET_DROP)==1):
                Utils._metrics.append(Constant._AWS_PACKET_DROP)
        
        if(Utils.is_valid(Utils._active_metric_provider)):
            Utils._logger.info(Constant._INFO_ACTIVE_METRIC_PROVIDER_FOUND.format(Utils._active_metric_provider.upper())) 
        count = 0    
        
        if(Utils._config.get(Constant._AZURE_METRIC)==1):
            count = count + 1
        if(Utils._config.get(Constant._VMWARE_METRIC)==1):
            count = count + 1    
        if(Utils._config.get(Constant._AWS_METRIC)==1):
            count = count + 1
            
        if(count > 1):
            Utils._logger.warn(Constant._WARN_MULTIPLE_METRIC_FOUND)  
            
    @staticmethod
    def validate():
        status=True
        if(Utils._thunders is None or Utils._thunders.get(Constant._THUNDERS) is None or Utils._thunders.get(Constant._THUNDERS)[0] is None or Utils._config is None or Utils._main is None):
            return False

        if(not Utils.is_valid(Utils._active_log_provider) and not Utils.is_valid(Utils._active_metric_provider)):
            return False
        
        if(Utils.is_valid(Utils._active_metric_provider) and  ((not Utils.is_valid(Utils._metrics) and not Utils.is_valid(Utils._non_l4v_metrics)) or (len(Utils._metrics) + len(Utils._non_l4v_metrics)) <=0)):
            Utils._logger.warn(Constant._ERROR_NO_VALID_METRIC_FOUND)
            
        if(Utils.is_valid(Utils._active_log_provider) or Utils.is_valid(Utils._active_metric_provider)): 
            status = True
            Utils.http_ssl_verify()
        else:
            status=False
        
        return status
        
    @staticmethod
    def date_time_utc():
        return datetime.datetime.now(datetime.timezone.utc)
     
    @staticmethod
    def is_valid(*arguments):
        valid=True
        for arg in arguments:
            if(arg is None or arg =="XXXX" or arg=="" or arg==" " or arg=="none" or arg=="None"):
                return False
        return valid;
    
    @staticmethod
    def is_integer(n):
        try:
            float(n)
        except ValueError:
            return False
        else:
            return float(n).is_integer()

    @staticmethod
    def map_to_value(content, file):
        
        if(content):
            if(not Utils.is_valid(file)):
                file = {}
                for line in content.split("\n"):
                    if not line:
                        continue
                    if "=" in line:
                        key, val = line.split("=",1)
                        file[key.strip()] = val.strip()
        return file
    
    @staticmethod
    def load_configuration(path, variable, error, mode):
        if(not Utils.is_valid(variable) and Utils.is_valid(path)):
            variable = None
            try:
                # JSON file
                con = open (path.strip(' \t\n\r'), "r", encoding=Constant._ENCODING)
                
                if(mode==Constant._KEYVALUE):
                    # Reading from key/value file
                    variable = Utils.map_to_value(con.read(), variable)
                elif(mode==Constant._JSON):
                    # Reading from json file
                    variable = json.loads(con.read())
                con.close()
            except Exception as exp:
                Utils._logger.error((Constant._ERROR_FILE_NOT_FOUND.format(path)) + error)
                Utils._logger.debug(exp)
            return variable
     
    @staticmethod
    def load_temp_storage(file):
        try:
            with open(Constant._TEMP_CONF_PATH + file, "r", encoding="utf-8") as con:
                variable = json.loads(con.read())
            con.close()
        except Exception:
            variable = {}
        return variable
    
    
    @staticmethod
    def write(variable, file):
        try:
            asyncio.run(Utils.async_write(variable, file))
        except Exception:
            Utils.sync_write(variable, file)
        
    @staticmethod
    async def async_write(variable, file):
        data = Utils.load_temp_storage(file)
        for key in variable.keys():
            data[key]=variable[key]
        with open(Constant._TEMP_CONF_PATH + file, "w+", encoding="utf-8") as con:
                    json.dump(data, con, default=str)
        con.close()
        
    @staticmethod
    def sync_write(variable, file):
        data = Utils.load_temp_storage(file)
        for key in variable.keys():
            data[key]=variable[key]
        with open(Constant._TEMP_CONF_PATH + file, "w+", encoding="utf-8") as con:
                    json.dump(data, con, default=str)
        con.close()
        
        
               
    @staticmethod
    def get_headers(token):
        _headers = {
            Constant._AUTHORIZATION: token,
            Constant._CONTENT_TYPE: Constant._APPLICATION_JSON,
            Constant._ACCEPT: Constant._APPLICATION_JSON 
        }
        return _headers
    
    @staticmethod
    def rest_api(url, headers, body, method, error, ip):
        try:
            response = requests.request(method, url,
                                        headers=headers, data=body, verify=Utils._http_ssl_verify,
                                        timeout=(int(Utils._main.get(Constant._API_TIMEOUT))))
            if response.status_code != 200 and response.status_code != 201 and response.status_code != 202 and response.status_code != 203 and response.status_code != 204:
                Utils._logger.error((Constant._ERROR_REQUEST_FAILED+error).format(url,response.json() if Utils.is_valid(response.text) else "{}" ,ip))
                return None
            return response
        except (ConnectionError, ConnectTimeout, ReadTimeout) as con:
            Utils._logger.error((Constant._ERROR_SERVER_NOT_REACHABLE).format(url,con))
            return None
        except (OSError) as os:
            Utils._logger.error((Constant._ERROR_SSL_NOT_REACHABLE+error).format(url,os,ip))
            return None
        except Exception as exp:
            Utils._logger.error((Constant._ERROR_REQUEST_FAILED+error).format(url,exp,ip))
            return None 
            
    @staticmethod
    def priority(log):
        if(" Info " in log):
            return "Info"
        elif(" Warning " in log):
            return "Warning"
        elif(" Error " in log):
            return "Error"
        elif(" Notice " in log):
            return "Notice"
        elif(" Debug " in log):
            return "Debug"
        else:
            return "Info"
                
    @staticmethod
    def http_ssl_verify():
        Utils._http_ssl_verify = Utils._main.get(Constant._SSL_VERIFY, "True")
        if(Utils._http_ssl_verify=="False"):
            Utils._http_ssl_verify = False
        elif(Utils.is_valid(Utils._http_ssl_verify)=="True"):
            Utils._http_ssl_verify = True
        else:
            Utils._http_ssl_verify = Utils._main.get(Constant._SSL_VERIFY)
            
        
        
        
        


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
VMWare Handler to connect and publish the data.
To use, simply 'import VMWareHandler' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com); Vikas (vgautam@a10networks.com)'


import json
from common.toa_constant import Constant
from common.toa_utils import Utils
from common.toa_logging import Logging

class VMWareHandler:
    
    _logger = Logging().get_logger("VMWareHandler")
    _credentials=None
    _thunder=None
    _token=None
    _url=None
        
    def __init__(self, thunder, token):
        self._thunder=thunder
        self._credentials =  Utils._vmware_credentials 
        self._token=token
 
    def token(self):
        _body = json.dumps({
                Constant._USERNAME: self._credentials.get(Constant._VMWARE_VROPS_USERNAME),
                Constant._PASSWORD: self._credentials.get(Constant._VMWARE_VROPS_PASSWORD)
        })
        _headers = {
            Constant._CONTENT_TYPE: Constant._APPLICATION_JSON,
            Constant._ACCEPT: Constant._APPLICATION_JSON 
        }
        response = Utils.rest_api((Constant._VMWARE_BASE_URL+"/auth/token/acquire").format(Utils._config.get(Constant._VMWARE_VROPS_HOST)), _headers, _body, Constant._POST, Constant._ERROR_VMWARE_AUTH_RESPONSE, self._thunder.get(Constant._IP))
        self._token = response.json()['token']
        return self._token
        
    def publish_metric(self, data):

        
        if(Utils.is_valid(data) and len(data) > 0):
            adaptor = "ThunderAdaptor"
            metric_data = {"stat-content": []}
            for metric in data:
                metric_dir = {
                    "statKey": self._thunder.get(Constant._NAMESPACE) +"|%s" % metric,
                    "timestamps": [Utils._datetime_epoc],
                    "data": [data[metric]]
                }
                metric_data['stat-content'].append(metric_dir)
            response = Utils.rest_api((Constant._VMWARE_BASE_URL+"/resources/{}/stats/adapterkinds/{}").format(Utils._config.get(Constant._VMWARE_VROPS_HOST), self._thunder.get(Constant._RESOURCE_ID), adaptor), Utils.get_headers(''.join(['vRealizeOpsToken ', self._token])), json.dumps(metric_data), Constant._POST, Constant._ERROR_VMWARE_VROPS_METRIC, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if (Utils.is_valid(response)):
                self._logger.info(Constant._SUCCESS_METRIC.format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper(), len(data), data))
                return Constant._SUCCESS
        else:
            self._logger.info((Constant._SKIP_METRIC).format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper()))
        return None 
    
    def format(self, data): 
        if isinstance(data, str):
            data = json.loads(data)

        # replace key log-data with text
        # add meta data to search logs on vRLI UI
        for each_log in data:
            if 'log-data' not in each_log:
                pass
            else:
                each_log['text'] = each_log.pop('log-data')
            each_log['fields'] = [
                {
                    "name": "LOG_TYPE",
                    "content": Constant._SYSLOG
                },
                {
                    "name": "APPNAME",
                    "content": Constant._THUNDER
                },
                {
                    "name": "HOSTNAME",
                    "content": self._thunder.get(Constant._RESOURCE_ID)
                },
                {
                    "name": "IP",
                    "content": self._thunder.get(Constant._PRIVATE_IP) if Utils.is_valid(self._thunder.get(Constant._PRIVATE_IP)) else self._thunder.get(Constant._IP)
                },
                {
                    "name": "AGENT",
                    "content": Constant._TOA
                },
                {
                    "name": "JOBID",
                    "content": Utils._datetime_id
                },
                {
                    "name": "PRIORITY",
                    "content": Utils.priority(each_log['text'])
                },
                {
                    "name": "PARTITION",
                    "content": self._thunder.get(Constant._PARTITION).upper()
                }
            ]
            yield each_log

    def publish_log(self, data):
        if(len(data)>0):
            generator = self.format(data)
            logs = []
            for log in generator:
                logs.append(log)
            body = {
                "events": logs
            }
            body = json.dumps(body)   
            response = Utils.rest_api(Constant._VMWARE_VRLI_URL.format(Utils._config.get('vmware_vrli_host'), self._thunder.get(Constant._PRIVATE_IP) if Utils.is_valid(self._thunder.get(Constant._PRIVATE_IP)) else self._thunder.get(Constant._IP)), Utils.get_headers(None), body, Constant._POST, Constant._ERROR_VMWARE_VRLI_LOGS, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]")
            if (response):
                self._logger.info(Constant._SUCCESS_LOG.format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper(), len(logs)))
                return Constant._SUCCESS
        else:
            self._logger.info(Constant._SKIP_LOG.format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper()))
        return None


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
AWS Handler to connect and publish teh data.
To use, simply 'import AWSHandler' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com); Pramod Nimbhore (pnimbhore@a10networks.com)'

import datetime
import boto3
from common.toa_constant import Constant
from common.toa_utils import Utils
from common.toa_logging import Logging

class AWSHandler:
    
    _logger = Logging().get_logger("AWSHandler")
    _config=None
    _credentials=None
    _temp=None
    _thunder=None
    _client=None
    _service=None
        
    def __init__(self, thunder, service):
        self._thunder = thunder 
        self._config =  Utils._config
        self._credentials =  Utils._aws_credentials 
        self._service= service
        self.init_client()
    
    def init_client(self):
        self._client = boto3.client(self._service, aws_access_key_id=self._credentials.get(Constant._AWS_ACCESS_KEY_ID),aws_secret_access_key=self._credentials.get(Constant._AWS_SECRET_ACCESS_KEY),region_name=self._config.get(Constant._AWS_REGION))
             
    def publish_metric(self, data):
        if(Utils.is_valid(data) and len(data) > 0):
            ip = self._thunder.get(Constant._PRIVATE_IP) if Utils.is_valid(self._thunder.get(Constant._PRIVATE_IP)) else self._thunder.get(Constant._IP)
            dimension = "%s (%s)" % (self._thunder[Constant._RESOURCE_ID], ip)
            # create a metric_data list
            metric_data = []
            # add metric data to metric_data list
            for metric in data:
                metric_data.append(
                    {
                        'MetricName': self._thunder.get(Constant._NAMESPACE),
                        'Dimensions': [
                            {
                                'Name': metric,
                                'Value': dimension
                            },
                        ],
                        'Timestamp': Utils._datetime, 
                        'StatisticValues': {
                            'SampleCount': 1,
                            'Sum': data[metric],
                            'Minimum': 0,
                            'Maximum': 0
                        },
                        'Unit': 'Percent',
                        'StorageResolution': 60
                    })
            try:
                # send thunder metrics to cloudwatch
                response = self._client.put_metric_data(Namespace=self._thunder.get(Constant._NAMESPACE),
                                                           MetricData=metric_data)
                if(Utils.is_valid(response, response.get("ResponseMetadata"), response.get("ResponseMetadata").get("HTTPStatusCode")) and (response.get("ResponseMetadata").get("HTTPStatusCode")==200 or response.get("ResponseMetadata").get("HTTPStatusCode")==201)):
                    self._logger.info(Constant._SUCCESS_METRIC.format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper(), len(data), data))
                    return Constant._SUCCESS
            except Exception as exp:
                self._logger.error((Constant._ERROR_AWS_CLOUDWATCH_METRICS).format(Constant._NAMESPACE, exp, data, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]"))
        else:
            self._logger.info((Constant._SKIP_METRIC).format(self._thunder.get(Constant._IP) , self._thunder.get(Constant._NAMESPACE).upper()))
        return None
    
    def publish_log(self, data):
        if(Utils.is_valid(data) and len(data) > 0):
            self.read()
            # get sequence token from temp file
            seq_token = self._temp.get("aws_nexttoken", " ")
            # get new log stream name
            log_stream_name, seq_token_status = self.stream()
            # get 13 digit timestamp
            timestamp = int(datetime.datetime.timestamp(Utils._datetime) * 1000)
            log_list = []
            #thunder_logs = eval(data)
    
            for log in data:
                log_list.append({'timestamp': timestamp, 'message': log["log-data"]}, )
            # create log event
            log_event = {'logGroupName': Utils._config.get(Constant._AWS_LOG_GROUP_NAME),
                         'logStreamName': log_stream_name, 'logEvents': log_list,
                         }
            if seq_token_status is not False:
                log_event['sequenceToken'] = seq_token
            try:
                # send logs to cloudwatch
                response = self._client.put_log_events(**log_event)
                # update config file
                self._temp["aws_nexttoken"] = response['nextSequenceToken']
                
                self._logger.info(Constant._SUCCESS_LOG.format(self._thunder.get(Constant._IP), self._thunder.get(Constant._NAMESPACE).upper(), len(data)))
                self.write()
                return Constant._SUCCESS
            except Exception as exp:
                self._logger.error((Constant._ERROR_AWS_CLOUDWATCH_LOGS).format(Utils._config.get(Constant._AWS_LOG_GROUP_NAME), exp, self._thunder.get(Constant._IP) + " [" +self._thunder.get(Constant._NAMESPACE)+"]"))
        else:
            self._logger.info((Constant._SKIP_LOG).format(self._thunder.get(Constant._IP) , self._thunder.get(Constant._NAMESPACE).upper()))
        return None
    
    def read(self):
        self._temp = Utils.load_temp_storage(self._thunder.get(Constant._IP)+"-"+self._thunder.get(Constant._PARTITION))
    
    def write(self):
        Utils.write(self._temp, self._thunder.get(Constant._IP)+"-"+self._thunder.get(Constant._PARTITION))
    
    def stream(self):
        try:
            seq_token_status = True
            ip = self._thunder.get(Constant._PRIVATE_IP) if Utils.is_valid(self._thunder.get(Constant._PRIVATE_IP)) else self._thunder.get(Constant._IP)
            #TODO
            today = Utils._datetime
            date1 = today.strftime("%d/%m/%Y")
            log_stream_name = "%s/%s/%s"%(str(date1), ip, self._thunder.get(Constant._NAMESPACE))
            if log_stream_name == self._temp.get(self._thunder[Constant._PARTITION] + "_log_stream", "") == "":
                return log_stream_name, seq_token_status
            try:
                self._client.create_log_stream(
                    logGroupName=Utils._config.get(Constant._AWS_LOG_GROUP_NAME),
                    logStreamName=log_stream_name)
            except Exception:
                return log_stream_name, seq_token_status
            seq_token_status = False
            self._temp[self._thunder[Constant._PARTITION] + "_log_stream"] = log_stream_name
            return log_stream_name, seq_token_status
        except Exception as exp:
            self._logger.error((Constant._ERROR_AWS_LOGSTREAM).format(exp, self._thunder[Constant._IP]))
        return None

    def collect_thunders_from_autoscale(self):
        try:
            user_name = self._thunder.get(Constant._USERNAME, "admin")
            resource_id = self._thunder.get(Constant._RESOURCE_ID)
            thunder_password = self._thunder.get(Constant._PASSWORD)
            ec2 = boto3.resource(Constant._AWS_BOTO3_SERVICE_EC2)
            response = self._client.describe_auto_scaling_groups(AutoScalingGroupNames=[resource_id])
            groups = response.get("AutoScalingGroups")
            if len(groups) > 0:
                instances = (groups[0].get('Instances'))
                thunders = [
                    {Constant._IP: ec2.Instance(instance.get('InstanceId')).private_ip_address,
                    Constant._USERNAME: user_name, 
                    Constant._PASSWORD: thunder_password,
                    Constant._PARTITION: self._thunder.get(Constant._PARTITION, "SHARED"),
                    Constant._RESOURCE_ID: instance.get('InstanceId')} for instance in instances]
                
                if(Utils.is_valid(thunders) and len(thunders) >= 0):
                    return thunders
        except Exception as exp:
            self._logger.error((Constant._ERROR_AWS_AUTOSCALE_GROUP_LOGS).format(self._thunder.get(Constant._RESOURCE_ID), exp))
        return None
    
    


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Processing engine to process metrics information from thunder to configured provider.
To use, simply 'import Metric' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from common.toa_constant import Constant
from common.toa_utils import Utils
from collections import ChainMap
import concurrent.futures
from common.toa_logging import Logging
from handler.thunder_handler import ThunderHandler
from handler.aws_handler import AWSHandler
from handler.vmware_handler import VMWareHandler
from handler.azure_handler import AzureHandler
from concurrent.futures import ThreadPoolExecutor

class MetricProcessor:
    
    _logger = Logging().get_logger("MetricProcessor")
    _metric=None
    _type=None
      
    def __init__(self, metric, partition):
        self._metric=metric
        self._type=partition
    
    def process(self, thunder):
        if(Utils.is_valid(Utils._active_metric_provider) and Utils.is_valid(self._metric) and len(self._metric) >0):
            _thunder = dict(thunder._thunder)
            if Utils._active_metric_provider==Constant._AWS_METRIC:
                self.process_aws_metric(_thunder, thunder)
            elif Utils._active_metric_provider==Constant._AZURE_METRIC:
                self.process_azure_metric(_thunder, thunder)
            elif Utils._active_metric_provider==Constant._VMWARE_METRIC:
                self.process_vmware_metric(_thunder, thunder)
    
    def process_aws_metric(self, _thunder, thunder):
        _metrics = self.collect_metric(_thunder, thunder, None)
        if(Utils.is_valid(_metrics)):
            AWSHandler(_thunder, Constant._AWS_BOTO3_SERVICE_CLOUDWATCH).publish_metric(_metrics)
       
    def process_vmware_metric(self, _thunder, thunder): 
        _metrics = self.collect_metric(_thunder, thunder, None)
        if(Utils.is_valid(_metrics)):
            handler = VMWareHandler(_thunder, None)
            handler.token()
            handler.publish_metric(_metrics)
       
    def process_azure_metric(self, _thunder, thunder): 
        handler = AzureHandler(_thunder, None)
        handler.token(Constant._AZURE_MONITORING_RESOURCE)
        handler.log(self.collect_metric(_thunder, thunder, handler.publish_metric))
        
    def collect_metric(self, _thunder,  thunder, callback):
        metric_data = []
        thunders = [ThunderHandler(metric=metric, date=None, thunder=_thunder, token=thunder._token, callback=callback) for metric in self._metric]
        with ThreadPoolExecutor(max_workers=int(Utils._main.get(Constant._THREADPOOL_MAX_WORKERS))) as collect:
            futures = [collect.submit(thunder.collect_metric) for thunder in thunders]
            for future in concurrent.futures.as_completed(futures):
                if future.result() is not None:
                    metric_data.append(future.result())
            return dict(ChainMap(*metric_data))
     
    


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Processing engine to process individual thunder.
To use, simply 'import ThunderProcessor' and use it!
"""
from processor.partition_processor import PartitionProcessor
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from common.toa_constant import Constant
from common.toa_utils import Utils
from common.toa_logging import Logging
from handler.thunder_handler import ThunderHandler
from concurrent.futures import ThreadPoolExecutor

class ThunderProcessor:
    
    _logger = Logging().get_logger("ThunderProcessor") 
    
    def process(self, thunder):
        try:
            thunders = []
            count = 0
            partitions = self.partitions(thunder)
            if(thunder.get(Constant._PARTITION)=="*"):
                thunder[Constant._PARTITION]= Constant._SHARED
                thunder[Constant._NAMESPACE]= Constant._THUNDER+"-"+Constant._SHARED.upper()
                thunders.append(dict(thunder))
                
                if(Utils.is_valid(partitions) and len(partitions) > 0):
                    for partition in partitions:
                        if(count == Constant._PARTITION_LIMIT - 1):
                            break
                        thunder[Constant._PARTITION]=partition
                        thunder[Constant._NAMESPACE]= Constant._THUNDER+"-"+partition.upper()
                        thunders.append(dict(thunder))
                        count+=1
            else:
                partitions_upper = []
                if(Utils.is_valid(partitions)):
                    partitions_upper = [part.upper() for part in partitions]
                for partition in thunder.get(Constant._PARTITION):
                        if(count == Constant._PARTITION_LIMIT):
                            break
                        if(Utils.is_valid(partitions) and partition.strip().upper() in partitions_upper):
                            name = partitions[partitions_upper.index(partition.strip().upper())]
                            thunder[Constant._PARTITION]=name
                            thunder[Constant._NAMESPACE]= Constant._THUNDER+"-"+name.upper()
                            thunders.append(dict(thunder))
                            count+=1
                        elif(partition.strip().upper()==Constant._SHARED.upper()):
                            thunder[Constant._PARTITION]= Constant._SHARED
                            thunder[Constant._NAMESPACE]= Constant._THUNDER+"-"+Constant._SHARED.upper()
                            thunders.append(dict(thunder))
                        else:
                            self._logger.warn((Constant._WARN_PARTITION_NOT_FOUND).format(partition, thunder[Constant._IP])) 
                
                if(len(thunders)==0):
                    thunder[Constant._PARTITION]= Constant._SHARED
                    thunder[Constant._NAMESPACE]= Constant._THUNDER+"-"+Constant._SHARED.upper()
                    thunders.append(dict(thunder))                 
            
            thunder[Constant._PARTITION]= Constant._THUNDER
            thunder[Constant._NAMESPACE]= Constant._THUNDER
            thunders.append(dict(thunder))    
        
        
            with ThreadPoolExecutor(max_workers=int(Utils._main.get(Constant._THREADPOOL_MAX_WORKERS))) as executor:
                executor.map(PartitionProcessor().process, thunders)
        except Exception as exp:
            self._logger.debug(exp)
                     
    def partitions(self, thunder):
        handler = ThunderHandler(None, None, thunder, None, callback=None) 
        handler.token()
        if(not Utils.is_valid(handler._token)):
            return None

        partitions = handler.partitions()
        handler.signout()
        return partitions



"""
Setup script to setup thunder observability agent
Create By: Vikas Gautam
"""

import os
import shutil
import compileall
from sys import platform
from setuptools import setup, find_packages
from setuptools.command.install import install
from pathlib import Path

# check operating system and username
platform = "linux"

# compile all python files
source_code_path = "thunder-observability-agent"
compileall.compile_dir(source_code_path, force=True)

# package metadata
if platform == "win32" or platform == "win64":
    package_dir_loc = {"": ".\\"}
elif platform == "linux":
    package_dir_loc = {"": "./"}
else:
    package_dir_loc = {"": ""}


this_directory = Path(__file__).parent
readme = (this_directory / "README.md").read_text()

setup(
    name="thunder-observability-agent",
    version="1.0.0",
    description="A10 Thunder Observability Agent is a lightweight autonomous data processing engine that can be externally installed and configured for multiple A10 Thunder Instances to collect, process and publish performance metrics and logs into multiple monitoring dashboard.",
    keywords=["Python","A10Networks","Observability","Agent","Thunder","vThunder","cThunder","Thunder","AWS","Azure","Cloudwatch","VMware","Log","Insight","Application","App","Insight","Analytics","Dashboard","Monitoring","VROPS","VRLI","Operations","A10"],
    long_description=readme,
    long_description_content_type='text/markdown',
    Documentation="https://documentation.a10networks.com",
    Repository="https://github.com/a10networks/thunder-observability-agent/tree/release/v1.0.0",
    Changelog="https://github.com/a10networks/thunder-observability-agent/tree/release/v1.0.0/README.md",
    dependencies=[
      "crontab",
      "rsyslog"
      "requests>=2.27.1",
      "boto3>=1.23.10"
    ],
    # requires-python=">=3.6",
    url="https://github.com/a10networks/thunder-observability-agent",
    # authors=[{"name" == "Dinesh Kumar Lohia", "email" == "dlohia@a10networks.com"}],
    author="Dinesh Kumar Lohia",
    author_email="dlohia@a10networks.com",
    maintainer="A10 Networks",
    maintainer_email="support@a10networks.com",
    # maintainers = [{"name" == "Dinesh Kumar Lohia", "email" =="dlohia@a10networks.com"}],
    license="THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT.",
    packages=["thunder-observability-agent", "thunder-observability-agent.common",
              "thunder-observability-agent.handler", "thunder-observability-agent.processor", "thunder-observability-agent.config"],
    include_package_data=True,
    data_files=[("thunder-observability-agent", ["thunder-observability-agent/config.json",
                                                 "thunder-observability-agent/logging.conf",
                                                 "thunder-observability-agent/main.properties",
                                                 "thunder-observability-agent/init.sh"]),
                ("thunder-observability-agent/.tmp/config/.aws", ["thunder-observability-agent/config/.aws/config",
                                                                  "thunder-observability-agent/config/.aws/credentials"]),
                ("thunder-observability-agent/.tmp/config/.azure", ["thunder-observability-agent/config/.azure/credentials"]),
                ("thunder-observability-agent/.tmp/config/.vmware", ["thunder-observability-agent/config/.vmware/credentials"]),
                ("thunder-observability-agent/.tmp/config/.thunder", ["thunder-observability-agent/config/.thunder/credentials"])
                ],
    package_dir=package_dir_loc,
    package_data={"thunder-observability-agent": ["*.py", "*.md", "*.txt", "pdf"],
                  "thunder-observability-agent.common": ["*.py"],
                  "thunder-observability-agent.handler": ["*.py"],
                  "thunder-observability-agent.processor": ["*.py"]},
    install_requires=["requests>=2.27.1", "boto3>=1.23.10"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: Other/Proprietary License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10"
        ]
)



#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT

"""
Configure constants values.
To use, simply 'import Constant' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

import os
import sys
import warnings

class Constant:

    sys.path.append(os.path.dirname(os.path.abspath(__file__)))  
    warnings.filterwarnings('ignore')
    
    _DEFAULT_TOA_PACKAGE_PATH = "/usr/toaenv/thunder-observability-agent"
    _TOA_CONFIG_PATH = os.environ.get("TOA_CONFIG_PATH", _DEFAULT_TOA_PACKAGE_PATH)
    _LOGGING_CONF_PATH = _TOA_CONFIG_PATH + "/logging.conf"
    _MAIN_CONF_PATH = _TOA_CONFIG_PATH + "/main.properties"
    _TEMP_CONF_PATH = _TOA_CONFIG_PATH + "/"
    _CONFIG_PATH = "config_path"
    _AWS_CREDENTIALS_PATH = "aws_credentials_path"
    _AWS_CONFIG_PATH = "aws_config_path"
    
    _AZURE_CREDENTIALS_PATH = "azure_credentials_path"
    _VMWARE_CREDENTIALS_PATH = "vmware_credentials_path"
    _THUNDER_CREDENTIALS_PATH = "thunder_credentials_path"
    _LOG_DELAY_MIN="log_collection_delay_min"
    _CRON_DELAY_MIN="cron_job_frequency_min"
    _THREADPOOL_MAX_WORKERS="max_threads"
    _API_TIMEOUT="http_connection_timeout_sec"
    _SSL_VERIFY="http_ssl_verify"
    _USERNAME="username"
    _PASSWORD="password"
    _IP="ip"
    _PRIVATE_IP="private_ip"
    _RESOURCE_ID="resource_id"
    _CREDENTIALS="credentials"
    _AUTHORIZATION="Authorization"
    _DATA="data"
    _CLOUD="cloud"
    
    _VMWARE_VROPS_HOST="vmware_vrops_host"
    _VMWARE_VRLI_HOST="vmware_vrli_host"
    _VMWARE_VROPS_USERNAME="vmware_vrops_username"
    _VMWARE_VROPS_PASSWORD="vmware_vrops_password"
    
    _AWS_ACCESS_KEY_ID="aws_access_key_id"
    _AWS_SECRET_ACCESS_KEY="aws_secret_access_key"
    _AWS_REGION="region"
    _AWS_METRIC_NAMESPACE="aws_metric_namespace"
    _AWS_LOG_GROUP_NAME="aws_log_group_name"
    _AWS_OUTPUT="output"
    
    _THUNDER="THUNDER"
    _THUNDERS="thunders"
    _SYSLOG="Syslog"
    _TOA="TOA"
    _NAMESPACE="namespace"
    
    _PARTITION_NAME="partition-name"
    #_ACTIVE_PARTITIONS="active_partitions"
    _PARTITION="active_partitions"
    _SHARED="shared"
    _PARTITION_LIMIT=20
    _DELIMETER=","
    _NON_L3V="non-l3v"
    _L3V="l3v"
    
    
    _AZURE_CLIENT_ID="azure_client_id"
    _AZURE_SECRET_ID="azure_secret_id"
    _AZURE_TENANT_ID="azure_tenant_id"
    _AZURE_LOG_WORKSPACE_ID="azure_log_workspace_id"
    _AZURE_WORKSPACE_PRIMARY_KEY="azure_workspace_primary_key"
    _AZURE_LOCATION="azure_location"
    _AZURE_METRIC_RESOURCE_ID="azure_metric_resource_id"
    
    
    _ENCODING = "utf-8"
    _KEYVALUE="keyvalue"
    _JSON="json"
    _AWS_THUNDER_AUTOSCALABLE="aws_thunder_auto_scalable"
    _AZURE_THUNDER_AUTOSCALABLE="azure_thunder_auto_scalable"
    _VMWARE_THUNDER_AUTOSCALABLE="vmware_thunder_auto_scalable"
    
    
    _ERROR_FILE_NOT_FOUND       = "Error           : File not found or corrupt. Please check file and path: [{}]. "
    _ERROR_REQ_PARAM_NOT_FOUND  = "Error           : Required params not found. " 
    _DEFAULT_PATH               = "Default file path is /usr/toaenv/thunder-observability-agent or check TOA_CONFIG_PATH environment variable."
    _ERROR_SERVER_NOT_REACHABLE = "Error           : Server connection timeout. [URI: {}, Trace: {}]. Please ping ip or check network settings."
    _ERROR_SSL_NOT_REACHABLE    = "Error           : TLS secure connection error. Please check network settings or param [http_ssl_verify] options in documentation. [URI: {}, Trace: {}]. "
    _ERROR_REQUEST_FAILED       = "Error           : Request failed. [URI: {}, Trace: {}]. "
    _ERROR_TASK_ABORT           = " Task abort for thunder: {}."
    

    _ERROR_MAIN_NOT_FOUND               ="Main properties not found. " + _DEFAULT_PATH
    _ERROR_CONFIG_NOT_FOUND             ="Application config not found. Please check [config_path] in main.properties. "
    _ERROR_AZURE_CREDENTIALS_NOT_FOUND  ="Azure credentials not found. Please check [azure_credentials_path] in main.properties. "
    _ERROR_THUNDER_CREDENTIALS_NOT_FOUND="Thunder credentials not found. Please check [thunder_credentials_path] in main.properties. "
    _ERROR_AWS_CREDENTIALS_NOT_FOUND    ="AWS credentials not found. Please check [aws_credentials_path] in main.properties. "
    _ERROR_AWS_CONFIG_NOT_FOUND         ="AWS config not found. Please check [aws_config_path] in main.properties. "
    _ERROR_VMWARE_CREDENTIALS_NOT_FOUND ="VMWare credentials not found. Please check [vmware_credentials_path] in main.properties. "
    _ERROR_AWS_CRED_NOT_FOUND           =_ERROR_REQ_PARAM_NOT_FOUND + "Please check [aws_access_key_id, aws_secret_access_key] in aws credentials. Metric/log publish task abort."
    

    _ERROR_AZURE_CRED_AUTO_METRIC_NOT_FOUND  = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [azure_client_id, azure_secret_id, azure_tenant_id] in azure credentials. Collect thunders from autoscale task abort."
    _ERROR_AZURE_CREDENTIALS_METRIC_NOT_FOUND= _ERROR_REQ_PARAM_NOT_FOUND + "Please check [azure_client_id, azure_secret_id, azure_tenant_id, azure_location] in azure credentials. Metric publish task abort."
    _ERROR_AZURE_CREDENTIALS_LOG_NOT_FOUND   = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [azure_workspace_primary_key] in azure credentials. Log publish task abort."
    _ERROR_AZURE_CONFIG_METRIC_NOT_FOUND     = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [azure_metric_resource_id] in config.json. Metric publish task abort."
    _ERROR_AZURE_CONFIG_LOG_NOT_FOUND        = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [azure_log_workspace_id] in config.json. Log publish task abort."
    _ERROR_AWS_CONFIG_METRIC_NOT_FOUND       = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [region] in aws config. Metric/Log publish task abort."
    _ERROR_AWS_CONFIG_LOG_NOT_FOUND          = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [aws_log_group_name] in config.json. Log publish task abort."
    _ERROR_VMWARE_CONFIG_METRIC_NOT_FOUND    = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [vmware_vrops_host] in config.json. Metric publish task abort."
    _ERROR_VMWARE_CONFIG_LOG_NOT_FOUND    = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [vmware_vrli_host] in config.json. Log publish task abort."
    
    _ERROR_VMWARE_CRED_METRIC_NOT_FOUND      = _ERROR_REQ_PARAM_NOT_FOUND + "Please check [vmware_vrops_username, vmware_vrops_password] in vmware credentials. Metric publish task abort."
    
    
    _ERROR_NO_VALID_LOG_DELAY_IN_MIN_FOUND   = "WARNING       : Invalid value found for "+_LOG_DELAY_MIN  +": [{}]. Only positive integer value is allowed for example [0 or 1 or 2 or nth.]. Considering default value [0] for processing, please check and update in main.properties"
    _ERROR_NO_VALID_CRON_DELAY_IN_MIN_FOUND  = "WARNING       : Invalid value found for "+_CRON_DELAY_MIN +": [{}]. Only positive integer value is allowed for example [0 or 1 or 2 or nth.]. Considering default value [1] for processing, please check and update in main.properties"
    
    
    _ERROR_AWS_LOGSTREAM            = "Error           : Request failed [Trace: {}]. Unable to publish logstream into aws cloudwatch."            + _ERROR_TASK_ABORT
    _ERROR_AWS_CLOUDWATCH_LOGS      = "Error           : Request failed [LogGroupName: {}, Trace: {}]. Unable to publish log into aws cloudwatch."                  + _ERROR_TASK_ABORT
    _ERROR_AWS_CLOUDWATCH_METRICS   = "Error           : Request failed [Namespace: {}, Trace: {}]. Unable to publish metric: [{}] into aws cloudwatch."         + _ERROR_TASK_ABORT
    _ERROR_AWS_AUTOSCALE_GROUP_LOGS = "Error           : Request failed [resource_id: {}, Trace: {}]. Unable to fetch thunder instances from aws autoscale group."
   
    _ERROR_VMWARE_AUTH_RESPONSE = "Unable to fetch token. Please check [vmware_vrops_host, vmware_vrops_username, vmware_vrops_password] in config.json/vmware credentials." + _ERROR_TASK_ABORT
    _ERROR_AZURE_AUTH_RESPONSE=   "Unable to fetch token. Please check [azure_client_id, azure_secret_id, azure_tenant_id] in azure credentials." + _ERROR_TASK_ABORT
    _ERROR_THUNDER_AUTH_RESPONSE= "Unable to fetch token. Please check [username, password] in thunder credentials."  + _ERROR_TASK_ABORT
    
    _ERROR_THUNDER_MANG_IP_NOT_FOUND    = "Unable to fetch 'datetime' from thunder."                                + _ERROR_TASK_ABORT
    _ERROR_THUNDER_CLOCK_NOT_FOUND      = "Unable to fetch 'datetime' from thunder."                                + _ERROR_TASK_ABORT
    _ERROR_THUNDER_LOGOFF_RESPONSE      = "Unable to call 'logout' from thunder."                                  + _ERROR_TASK_ABORT
    
    _ERROR_METRIC_CPS_NOT_FOUND         = "Unable to fetch 'Total New Connection (Per Sec)' metric from thunder."   + _ERROR_TASK_ABORT
    _ERROR_METRIC_CPU_NOT_FOUND         = "Unable to fetch 'CPU Usage Percentage (Data)' metric from thunder."      + _ERROR_TASK_ABORT
    _ERROR_METRIC_MEMORY_NOT_FOUND      = "Unable to fetch 'Memory Usage Percentage' metric from thunder."          + _ERROR_TASK_ABORT
    _ERROR_METRIC_DISK_NOT_FOUND        = "Unable to fetch 'Disk Usage Percentage' metric from thunder. "           + _ERROR_TASK_ABORT
    _ERROR_METRIC_THROUGHPUT_NOT_FOUND  = "Unable to fetch 'Throughput Rate (Global/BPS)' metric from thunder."     + _ERROR_TASK_ABORT
    _ERROR_METRIC_INTERFACES_NOT_FOUND  = "Unable to fetch 'Interface Down Count (Data)' metric from thunder."      + _ERROR_TASK_ABORT
    _ERROR_METRIC_TPS_NOT_FOUND         = "Unable to fetch 'Transactions Rate (Sec)' metric from thunder."          + _ERROR_TASK_ABORT
    _ERROR_METRIC_SERVER_DOWN_NOT_FOUND = "Unable to fetch 'Server Down Count' metric from thunder."                + _ERROR_TASK_ABORT
    _ERROR_METRIC_SERVER_PER_NOT_FOUND  = "Unable to fetch 'Server Down Percentage' metric from thunder."           + _ERROR_TASK_ABORT
    _ERROR_METRIC_SSL_NOT_FOUND         = "Unable to fetch 'SSL Errors Count' metric from thunder."                 + _ERROR_TASK_ABORT
    _ERROR_METRIC_SERVER_ERROR_NOT_FOUND= "UnaRble to fetch 'Server Errors Count' metric from thunder."              + _ERROR_TASK_ABORT
    _ERROR_METRIC_SESSION_NOT_FOUND     = "Unable to fetch 'Total Session Count' metric from thunder."              + _ERROR_TASK_ABORT
    _ERROR_METRIC_PACKET_RATE_NOT_FOUND = "Unable to fetch 'Packet Rate (Sec)' metric from thunder."                + _ERROR_TASK_ABORT
    _ERROR_METRIC_PACKET_DROP_NOT_FOUND = "Unable to fetch 'Packet Drop Rate (Sec)' metric from thunder."           + _ERROR_TASK_ABORT
    
    _ERROR_PARTITION_NOT_FOUND = "Unable to fetch partition details from thunder."           + _ERROR_TASK_ABORT 
    _ERROR_PARTITION = "Unable to activate partition into thunder."           + _ERROR_TASK_ABORT 
    
    
    
    _ERROR_VMWARE_VROPS_METRIC              = "Unable to publish metric into vmware vrops."                   + _ERROR_TASK_ABORT
    _ERROR_VMWARE_VRLI_LOGS                 = "Unable to publish logs into vmware vrli."                            + _ERROR_TASK_ABORT
    _ERROR_AZURE_LOGS                       = "Unable to publish logs into azure log workspace."                    + _ERROR_TASK_ABORT
    _ERROR_AZURE_METRICS                    = "Unable to publish metric into azure insight/vm."               + _ERROR_TASK_ABORT
    _ERROR_AZURE_AUTOSCALE_THUNDERS         = "Unable to fetch thunder instances from azure autoscale [resource_id: {}]."
    
    _ERROR_NO_VALID_PROVIDER_FOUND          ="WARNING       : No provider is enabled for [metric: {}, log: {}]. To enable [provider: {} set to [1]] in config.json."
    _ERROR_NO_VALID_PROVIDER_CONFIG_FOUND   ="WARNING       : No log or metric is enabled. To enable [metric, log set to [1]] in config.json."
    _ERROR_NO_VALID_METRIC_FOUND            ="WARNING       : No metric enabled. To enable [cpu, memory, ...etc seto to [1]] in config.json."
    
    _ERROR_INVALID_THUNDER_AUTO             ="WARNING       : No thunder found for autoscale provider: {}, please check [autoscale[0/1], provider[aws/azure], username, password and resource_id] in thunder credentials. Thunder collection in autoscale mode task abort."
    _ERROR_INVALID_THUNDER_PARAMETERS       ="WARNING       : No thunder found, please check [ip, username, password and resource_id] in thunder credentials. Thunder collection task abort."
    
    _WARN_MULTIPLE_LOG_FOUND                ="WARNING       : Multiple log enabled,  please check [aws_log, azure_log, vmware_log] only one can be allowed."
    _WARN_MULTIPLE_METRIC_FOUND             ="WARNING       : Multiple metric enabled,  please check [aws_metric, azure_metric, vmware_metric] only one can be allowed."
    
    
    _WARN_ACTIVE_PARTITION_NOT_FOUND        ="WARNING       : No active partitions found for thunder [{}], setting to default 'SHARED'. Multiple L3V partition can be configured as comma separated for example if we have partition 'P1' and 'P2' then we can define as ['partition' : ' Shared,P1,P2'] upto "+str(_PARTITION_LIMIT)+" partitions."
    _WARN_PARTITION_NOT_FOUND               ="WARNING       : Provided partition [{}] not found in thunder."+_ERROR_TASK_ABORT
    _WARN_STANDBY_PARTITION                 ="WARNING       : Provided partition [{}] is in standby mode."+_ERROR_TASK_ABORT
    
   
    _AZURE_PROVIDER="azure_provider"
    _AZURE_METRIC="azure_metric"
    _AZURE_LOG="azure_log"
    
    _AUTOSCALE="autoscale"
    _PROVIDER="provider"
    _AZURE="azure"
    _AWS="aws"
    _VMWARE="vmware"
    
    _VMWARE_PROVIDER="vmware_provider"
    _VMWARE_METRIC="vmware_metric"
    _VMWARE_LOG="vmware_log"
    
    
    _AWS_PROVIDER="aws_provider"
    _AWS_METRIC="aws_metric"
    _AWS_LOG="aws_log"
    
    _AZURE_CPU="azure_cpu"
    _AZURE_MEMORY="azure_memory"
    _AZURE_DISK="azure_disk"
    _AZURE_THROUGHPUT="azure_throughput"
    _AZURE_INTERFACES="azure_interfaces"
    _AZURE_CPS="azure_cps"
    _AZURE_TPS="azure_tps"
    _AZURE_SERVER_DOWN_COUNT="azure_server_down_count"
    _AZURE_SERVER_DOWN_PERCENTAGE="azure_server_down_percentage"
    _AZURE_SSL_CERT="azure_ssl_cert"
    _AZURE_SERVER_ERROR="azure_server_error"
    _AZURE_SESSION="azure_sessions"
    _AZURE_PACKET_RATE="azure_packet_rate"
    _AZURE_PACKET_DROP="azure_packet_drop"
    
    _AWS_CPU="aws_cpu"
    _AWS_MEMORY="aws_memory"
    _AWS_DISK="aws_disk"
    _AWS_THROUGHPUT="aws_throughput"
    _AWS_INTERFACES="aws_interfaces"
    _AWS_CPS="aws_cps"
    _AWS_TPS="aws_tps"
    _AWS_SERVER_DOWN_COUNT="aws_server_down_count"
    _AWS_SERVER_DOWN_PERCENTAGE="aws_server_down_percentage"
    _AWS_SSL_CERT="aws_ssl_cert"
    _AWS_SERVER_ERROR="aws_server_error"
    _AWS_SESSION="aws_sessions"
    _AWS_PACKET_RATE="aws_packet_rate"
    _AWS_PACKET_DROP="aws_packet_drop"
    
    _AWS_BOTO3_SERVICE_LOGS="logs"
    _AWS_BOTO3_SERVICE_CLOUDWATCH="cloudwatch"
    _AWS_BOTO3_SERVICE_AUTOSCALING="autoscaling"
    _AWS_BOTO3_SERVICE_EC2="ec2"
    
    _VMWARE_CPU="vmware_cpu"
    _VMWARE_MEMORY="vmware_memory"
    _VMWARE_DISK="vmware_disk"
    _VMWARE_THROUGHPUT="vmware_throughput"
    _VMWARE_INTERFACES="vmware_interfaces"
    _VMWARE_CPS="vmware_cps"
    _VMWARE_TPS="vmware_tps"
    _VMWARE_SERVER_DOWN_COUNT="vmware_server_down_count"
    _VMWARE_SERVER_DOWN_PERCENTAGE="vmware_server_down_percentage"
    _VMWARE_SSL_CERT="vmware_ssl_cert"
    _VMWARE_SERVER_ERROR="vmware_server_error"
    _VMWARE_SESSION="vmware_sessions"
    _VMWARE_PACKET_RATE="vmware_packet_rate"
    _VMWARE_PACKET_DROP="vmware_packet_drop"
    
    _TEMP_LAST_PACKET_SENT_TIME = "last_packet_sent_time"
    _TEMP_LAST_PACKET_COUNT = "last_packet_count"
    
    _TEMP_LAST_PACKET_DROPPED_SENT_TIME = "last_packet_dropped_sent_time"
    _TEMP_LAST_PACKET_DROPPED_COUNT = "last_packet_dropped_count"
    
    
    
    _THUNDER_BASE_URL="https://{}/axapi/v3"
    _VMWARE_BASE_URL="https://{}/suite-api/api"
    _VMWARE_VRLI_URL="https://{}/api/v2/events/ingest/{}"
    #TODO
      
    _AZURE_METRICS_URL = "https://{}.monitoring.azure.com{}/metrics"
    _AZURE_OAUTH2_URL = "https://login.microsoftonline.com/{}/oauth2/token"
    _AZURE_LOGS_URL = "https://{}.ods.opinsights.azure.com/api/logs?api-version=2016-04-01"
    _AZURE_VIRTUALMACHINES_URL="https://management.azure.com{}/virtualMachines?api-version=2022-11-01"
    _AZURE_BASE_URL="https://management.azure.com/{}?api-version=2022-03-01"
    _AZURE_MONITORING_RESOURCE="https://monitoring.azure.com/"
    _AZURE_MANAGEMENT_RESOURCE="https://management.azure.com/"
  
    
    _CONTENT_TYPE="Content-Type"
    _ACCEPT="Accept"
    _APPLICATION_JSON="application/json"
    _APPLICATION_URLENCODED="application/x-www-form-urlencoded"
    
    _POST="POST"
    _GET="GET"
    _SUCCESS="success"

    _INFO_JOB_ID                         = "Job No           : {}."
    _INFO_JOB_START                      = "Job Start Time   : {}."
    _INFO_JOB_TIME                       = "Job Execution    : {} seconds."
    _INFO_JOB_END                        = "Job End Time     : {}"
    _INFO_ACTIVE_THUNDERS_COUNT          = "No of Thunders   : {} {}."
    _INFO_ACTIVE_PARTITIONS_COUNT        = "No of Partitions : {} [Count: {}] [{}]."
    _INFO_ACTIVE_PROVIDER_LOG            = "Log Provider     : {}."
    _INFO_ACTIVE_PROVIDER_METRIC         = "Metric Provider  : {}."
    _INFO_ACTIVE_LOG_PROVIDER_FOUND      = "Log              : {}."
    _INFO_ACTIVE_METRIC_PROVIDER_FOUND   = "Metric           : {}."
    _SUCCESS_METRIC                      = "Published Metric : {} {} [Count: {}] [{}]."
    _SUCCESS_LOG                         = "Published Log    : {} {} [Count: {}]."
    _SKIP_LOG                            = "Published Log    : {} {} [No Data Found]."
    _SKIP_METRIC                         = "Published Metric : {} {} [No Data Found]."
    _DOCS                                = "Documentation    : www.a10networks.com or https://github.com/a10networks/thunder-observability-agent."
    
    


#!/usr/bin/python
# Copyright 2023, A10 Networks Inc. All Rights Reserved.
# THUNDER OBSERVABILITY AGENT END USER SOFTWARE LICENSE AGREEMENT
"""
Configure utility methods for toa framework.
To use, simply 'import Helper' and use it!
"""
__author__ = 'Dinesh Kumar Lohia (DLohia@a10networks.com)'

from processor.thunder_processor import ThunderProcessor
from common.toa_logging import Logging 
from common.toa_constant import Constant
from common.toa_utils import Utils
from handler.aws_handler import AWSHandler
from handler.azure_handler import AzureHandler
from concurrent.futures import ThreadPoolExecutor
class Helper:
    
    _logger = Logging().get_logger("Helper")
    
    def __init__(self):
        pass
    
    @staticmethod
    def init():
        try:
            Utils.load_properties()
            Utils.load_config()
            Utils.load_collect_provider() 
            Helper.load_autoscale_thunders()
            Utils.load_publish_provider()
            if(Utils.validate()):
                Helper.process()
        except Exception as exp:
            Utils._logger.debug(exp)
            return False
        
    @staticmethod
    def process():
        try:
            with ThreadPoolExecutor(max_workers=int(Utils._main.get(Constant._THREADPOOL_MAX_WORKERS))) as executor:
                executor.map(ThunderProcessor().process, Utils._thunders.get(Constant._THUNDERS))
        except Exception as exp:
            Utils._logger.debug(exp)

    @staticmethod
    def load_autoscale_thunders():
        if(Utils.is_valid(Utils._thunders)):
            if(Utils.is_valid(Utils._thunders.get(Constant._AUTOSCALE)) 
               and Utils._thunders.get(Constant._AUTOSCALE)==1 
               and Utils.is_valid(Utils._thunders.get(Constant._PROVIDER)) 
               and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)) 
               and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)[0]) 
               and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._USERNAME), Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._PASSWORD), Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._RESOURCE_ID))):
                if(Utils._thunders.get(Constant._PROVIDER)==Constant._AWS):
                    if(not Utils.is_valid(Utils._aws_credentials)): 
                        Utils._aws_credentials =  Utils.load_configuration(Utils._main.get(Constant._AWS_CREDENTIALS_PATH), Utils._aws_credentials, Constant._ERROR_AWS_CREDENTIALS_NOT_FOUND, Constant._KEYVALUE) 
                    if(not Utils.is_valid(Utils._aws_config)): 
                        Utils._aws_config =  Utils.load_configuration(Utils._main.get(Constant._AWS_CONFIG_PATH), Utils._aws_config, Constant._ERROR_AWS_CONFIG_NOT_FOUND, Constant._KEYVALUE) 
                    
                    if(Utils.is_valid(Utils._aws_credentials) and Utils.is_valid(Utils._aws_config)): 
                        handler = AWSHandler(Utils._thunders.get(Constant._THUNDERS)[0], Constant._AWS_BOTO3_SERVICE_AUTOSCALING)
                        Utils._thunders[Constant._THUNDERS] = handler.collect_thunders_from_autoscale()
                    else:
                        if(not Utils.is_valid(Utils._aws_credentials)): 
                            Utils._logger.error(Constant._ERROR_AWS_CREDENTIALS_NOT_FOUND)
                        if(not Utils.is_valid(Utils._aws_config)): 
                            Utils._logger.error(Constant._ERROR_AWS_CONFIG_NOT_FOUND)
                             
                        Utils._azure_credentials = None
                        Utils._aws_config = None
                        Utils._thunders=None
                elif(Utils._thunders.get(Constant._PROVIDER)==Constant._AZURE):
                    if(not Utils.is_valid(Utils._azure_credentials)):
                        Utils._azure_credentials =  Utils.load_configuration(Utils._main.get(Constant._AZURE_CREDENTIALS_PATH), Utils._azure_credentials, Constant._ERROR_AZURE_CREDENTIALS_NOT_FOUND, Constant._KEYVALUE) 
                
                    if(Utils._azure_credentials and Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_CLIENT_ID))
                        and Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_SECRET_ID))
                        and Utils.is_valid(Utils._azure_credentials.get(Constant._AZURE_TENANT_ID))): 
                        handler = AzureHandler(Utils._thunders.get(Constant._THUNDERS)[0], None)
                        handler.token(Constant._AZURE_MANAGEMENT_RESOURCE)
                        Utils._thunders[Constant._THUNDERS] = handler.collect_thunders_from_autoscale()
                    else:
                        Utils._logger.error(Constant._ERROR_AZURE_CRED_AUTO_METRIC_NOT_FOUND) 
                        Utils._azure_credentials = None
                        Utils._thunders=None
                else:
                    Utils._logger.warn((Constant._ERROR_INVALID_THUNDER_AUTO).format(Utils._thunders.get(Constant._PROVIDER)))
                    Utils._thunders=None
            elif(Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)) and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)[0]) and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._IP), Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._USERNAME), Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._PASSWORD)) and Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._IP)=="127.0.0.1"):
                pass
            elif(Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)) and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)[0]) and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._IP), Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._USERNAME), Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._PASSWORD), Utils._thunders.get(Constant._THUNDERS)[0].get(Constant._RESOURCE_ID))):
                pass
            else:
                Utils._thunders=None
                Utils._logger.warn(Constant._ERROR_INVALID_THUNDER_PARAMETERS)
                
            if(Utils.is_valid(Utils._thunders) and Utils.is_valid(Utils._thunders.get(Constant._THUNDERS))):
                Utils._logger.info(Constant._INFO_ACTIVE_THUNDERS_COUNT.format(str(len(Utils._thunders.get(Constant._THUNDERS))), [value['ip'] for value in Utils._thunders.get(Constant._THUNDERS)] ))
                
                for thunder in Utils._thunders.get(Constant._THUNDERS):
                    if(Utils.is_valid(thunder.get(Constant._PARTITION)) and not thunder.get(Constant._PARTITION)=="*"):
                        selected_partitions = set([part.strip().upper() for part in thunder.get(Constant._PARTITION).split(Constant._DELIMETER)])
                
                        if(Utils.is_valid(selected_partitions) and len(selected_partitions) > 0):
                            thunder[Constant._PARTITION] = selected_partitions
                            Utils._logger.info(Constant._INFO_ACTIVE_PARTITIONS_COUNT.format(thunder.get(Constant._IP), str(len(selected_partitions)), selected_partitions))
                        else:
                            thunder[Constant._PARTITION]= [Constant._SHARED]
                            Utils._logger.warn((Constant._WARN_ACTIVE_PARTITION_NOT_FOUND).format(thunder.get(Constant._IP)))
                            Utils._logger.info(Constant._INFO_ACTIVE_PARTITIONS_COUNT.format(thunder.get(Constant._IP), "1", Constant._SHARED.upper()))
                    elif(thunder.get(Constant._PARTITION)=="*"):
                        Utils._logger.info(Constant._INFO_ACTIVE_PARTITIONS_COUNT.format(thunder.get(Constant._IP), "1", "*"))
                    else:
                        thunder[Constant._PARTITION]= [Constant._SHARED]
                        Utils._logger.warn((Constant._WARN_ACTIVE_PARTITION_NOT_FOUND).format(thunder.get(Constant._IP)))
                        Utils._logger.info(Constant._INFO_ACTIVE_PARTITIONS_COUNT.format(thunder.get(Constant._IP), "1", Constant._SHARED.upper()))
                
            else:
                Utils._logger.info(Constant._INFO_ACTIVE_THUNDERS_COUNT.format(str(0),""))
                
    